;*************************************************************************************
; Filename :  DEBUG675.INC
; Description : Include file for PIC12F675 debugging
; MCU : 	PIC12F675
; Hardware :	AX-12 PIC12F675 Activity board
; Written by : Natthapol Wongsoonthornchai
; Copyright 2003 Innovative Experiment Co.,Ltd.
;*************************************************************************************

	LIST	P=12F675,W=-302  
;*************************************  
; DEBUG REGISTER 
;***************************************
__PCL	EQU	0x02	; Every bank 
__PCLATH	EQU	0x0A
__STATUS	EQU	0x03 

     	CBLOCK	0x4A	; RAM 
          __CHAR0,__CHAR1,__CHAR2,__CHAR3,__CHAR4,__CHAR5,__DATA_OUT
          __SAVE_STATUS,__SAVE_W,__SAVE_PCLATH,__DELAY0,__DELAY1,__DELAY2,__COUNT8
          __TX_BUFFER,__BUFFER,__COUNT1,__COUNT2,__D_VAR1,__D_VAR2,__ADDR_REG	
   	ENDC
;************************************   
; MACRO   
;**********************************************	
DEBUG	MACRO	ASCII0,ASCII1,ASCII2,ASCII3,ASCII4,ASCII5,__ADDR_REG,__DELAY_REG

	MOVWF	__SAVE_W	 ; Save W
	SWAPF	__PCLATH,0     ; Save PCLATH
	MOVWF	__SAVE_PCLATH
	SWAPF	__SAVE_PCLATH,1 
	CALL	__SAVE_REG     ; Save W,STATUS,PCLATH before debug		
	MOVLW	__DELAY_REG	 ; Load delay time
	MOVWF	__DELAY2

	MOVLW	ASCII0	 ; Send 6 ASCII
	MOVWF	__CHAR0
	MOVLW	ASCII1
	MOVWF	__CHAR1
	MOVLW	ASCII2
	MOVWF	__CHAR2
	MOVLW	ASCII3
	MOVWF	__CHAR3
	MOVLW	ASCII4
	MOVWF	__CHAR4
	MOVLW	ASCII5
	MOVWF	__CHAR5
	MOVFW	__ADDR_REG
              MOVWF         __DATA_OUT
	CALL	__DEBUGOUT
	ENDM		; END OF MACRO

;*************************************  
; TX & DELAY  
;*************************************
	ORG	.1023-.127

__SAVE_REG	SWAPF	__STATUS,0	; Save all flag except RP1,RP0
	MOVWF	__SAVE_STATUS	
	RETURN
__DEBUGOUT	BCF	0x05,1	; Clear GP1 latch
	MOVLW	.7
	MOVWF	0x19	; Comparator off
	BSF	0x03,5	; Bank 1	
	BCF	0x85,1	; GP1 = output
	BCF	0x9F,1	; GP1 = digital port
	CALL	0x3FF
	MOVWF	0x90	; Calibrated OSC.
	BCF	0x03,5	; Bank 0

	MOVFW	__CHAR0
	CALL	__SEND_DATA
	MOVFW	__CHAR1
	CALL	__SEND_DATA
	MOVFW	__CHAR2
	CALL	__SEND_DATA
	MOVFW	__CHAR3
	CALL	__SEND_DATA
	MOVFW	__CHAR4
	CALL	__SEND_DATA
	MOVFW	__CHAR5
	CALL	__SEND_DATA
	CALL	__SEND_EQUAL	; Send '=' 
	MOVLW	'$'	; Show $ for hexadecimal number
	CALL	__SEND_DATA
	MOVFW	__DATA_OUT    ; Debug register
	CALL	__SWAP_FILE    
	CALL	__SWAP_FILE	   

	CALL	__SEND_SPACE  ; Send '=' and ' '
	CALL	__SEND_EQUAL
	MOVLW	'%'	; Show % for binary number
	CALL	__SEND_DATA
	MOVLW	.8	; Rotate 8 times for showing binRY
	MOVWF	__COUNT8	
__ASCII4BIN	RLF	__DATA_OUT,1
	BC	__CARRY_SET
__CARRY_CLEAR	MOVLW	'0'
	GOTO	__LOGIC_0
__CARRY_SET	MOVLW	'1'
__LOGIC_0	CALL	__SEND_DATA
__CHECK8	DECFSZ	__COUNT8,1
	GOTO	__ASCII4BIN

	MOVLW	.10	   ; Send line feed
	CALL	__SEND_DATA
	MOVLW	.13	   ; Send carriage return
	CALL	__SEND_DATA
	GOTO	__DEBUG_DELAY    ; Return

__SWAP_FILE	MOVLW	B'00000011'	   ; Set up for look-up table
	MOVWF	__PCLATH	
	SWAPF	__DATA_OUT,1
	MOVFW	__DATA_OUT
	ANDLW	B'00001111'	   ; Mask bit
	CALL	__DEBUG_TABLE
	CALL	__SEND_DATA
	RETURN
__DEBUG_TABLE	ADDWF	__PCL,1	   ; BCD table
	DT	"0123456789ABCDEF" 

__SEND_DATA	MOVWF	__TX_BUFFER	; 8N1,9600 bps,not-invert
	MOVLW	.8
	MOVWF	__COUNT1
	BCF	0x05,1	; Start bit
	CALL	__DELAY94	
__LOOP_SEND	RRF	__TX_BUFFER,1	; Send LSB first
	BTFSC	0x03,0	; Skip if carry = 0
	GOTO	__SEND_1
__SEND_0	BCF	0x05,1	; GP1 = 0 for logic "0"
__LOGIC_1	CALL	__DELAY94	
	DECFSZ	__COUNT1,1
	GOTO	__LOOP_SEND		
	BSF	0x05,1	; Stop bit <HIGH>
	CALL	__DELAY94
	RETURN	
__SEND_1	BSF	0x05,1	; GP1 = 1 for logic "1"
	GOTO	__LOGIC_1
	
__DELAY94	MOVLW	.30	; 1 microsecond 	
	MOVWF	__D_VAR1	; 1 microsecond
	DECFSZ	__D_VAR1,1
	GOTO	$-1
	NOP		; 1 microsecond
	RETURN		; 2 microsecond,Total = 94 microsecond <delay 1 bit>
__DELAY51	MOVLW	.16	; 1 microsecond
	MOVWF	__D_VAR1	; 1 microsecond
	DECFSZ	__D_VAR1,1
	GOTO	$-1	
	RETURN		; 2 microsecond,TOTAL = 51 microsecond < delay 1/2 bit>
	
__SEND_EQUAL	MOVLW	'='
	CALL	__SEND_DATA
__SEND_SPACE	MOVLW	' '
	CALL	__SEND_DATA
	RETURN
__DEBUG_DELAY MOVF	__DELAY2,1	   ; Test fiel __DELAY2 = 0 ?	
	BZ	__RETURN	   ; Jump if not delay	
	CLRF	__DELAY0
	CLRF	__DELAY1
	DECFSZ	__DELAY0,1
	GOTO	$-1
	DECFSZ	__DELAY1,1
	GOTO	$-3
	DECFSZ	__DELAY2,1
	GOTO	$-5
__RETURN	MOVFW	__SAVE_PCLATH    ; Restore PCLATH
	MOVWF	__PCLATH		
	SWAPF	__SAVE_STATUS,0  ; Restore STATUS
	MOVWF	__STATUS
	SWAPF	__SAVE_W,1	   ; Restore W
	SWAPF	__SAVE_W,0	 
	BCF	0x03,5	   ; Return to bank 0
	RETURN
