;***************************************************************************
; Filename :  PROJ0202.ASM
; Description : Output port expansion for driving LED 7 segments 
; MCU : 	PIC12F675
; Hardware :	AX-12 PIC12F675 Activity board
; Written by :	Kridsada Jaiyen
; Copyright 2003 Innovative Experiment Co.,Ltd.
;***************************************************************************
	LIST      P=12F675,W=-302   	; list directive to define processor
	#INCLUDE <P12F675.INC>  	; processor specific variable definitions
	
	__CONFIG  0x01c4		; INTRC,WDT OFF,BROWNOUT OFF,MCLR OFF
	CBLOCK	0x20
	  DUMMY1,DUMMY2,COUNT,SEND,CNT
	ENDC

#DEFINE	DAT	GPIO,1		;***************************************
#DEFINE CLK	GPIO,4		; 74HC595 pin
#DEFINE LAT	GPIO,2		;*************************************** 

	ORG	0x000
	GOTO	INIT		; 7 Segment Table
	
TABLE	ADDWF	PCL,F
	DT	0x3F,0x06,0x5B,0x4F,0x66,0x6D,0x7D,0x07,0x7F,0x6F 

INIT   	MOVLW	.7          	; Comparator off
 	MOVWF	CMCON					
	BANKSEL	ANSEL 		; //Bank 1		
  	CLRF	ANSEL		; GP4,GP2,GP1,GP0 = digital I/O
	MOVLW	B'11101000' 	; GP4,GP2,GP1,GP0 = output   
	MOVWF	TRISIO
	BANKSEL	GPIO		; //Bank 0

;*******************************;
; Main Program 			;
;*******************************;		

MAIN	CLRF	CNT		; Start From 0	
AGAIN	MOVF	CNT,W		; Load Data
	CALL	TABLE		; Return Data From Table
	CALL	SEND8BIT	; Send Data To Shift Register	
	CALL	DELAY		; Long Time Delay	
	CALL	DELAY	
	CALL	DELAY	
	INCF	CNT,F		; Count Number
	movlw	0x0A		; Test Data More Than TEN
	SUBWF	CNT,W		; 
	BTFSS	STATUS,Z	; IF Data <> 10
	GOTO	AGAIN		; Next Count
	GOTO 	MAIN		; Else Start Again

;*******************************;
; Send Data 8 Bit To 74595	;
;*******************************;		
SEND8BIT MOVWF	SEND		; Load Data
	MOVLW	0x08		; Loop 8 Time : Send 8 Data Bit
	MOVWF	COUNT
CHK_BIT	BCF	DAT		; RB3 >> DATA PIN
	BTFSC	SEND,7		; Check bit 7 0 Or 1	
	BSF	DAT		; Send Bit 7 To Shift Register
CLOCK	BSF	CLK		; Clock Pulse
	BCF	CLK		
ROTATE	RLF	SEND,F		; Rotate Bit For Send Other Bit
	DECFSZ	COUNT,F		; Test Loop 8 Time ?
	GOTO	CHK_BIT		; If No Send Bit Again
LATCH	BSF	LAT		; Latch Data For Show Long Time
	BCF	LAT
	RETURN
	
;*******************************;
; Delay For Show Data		;
;*******************************;

DELAY	CLRF	DUMMY1		
DELAY1	CLRF	DUMMY2
	DECFSZ	DUMMY2,F
	GOTO	$-1
	DECFSZ	DUMMY1,F
	GOTO	DELAY1
	RETURN
	
	END                    	; directive 'end of program'

