;***************************************************************************
; Filename :  PROJ03.ASM
; Description : 9-minute timer by Timer1 
; MCU : 	PIC12F675
; Hardware :	AX-12 PIC12F675 Activity board
; Copyright 2003 Innovative Experiment Co.,Ltd.
;***************************************************************************
 
 	TITLE  	"TIMER 9 MINUTES"
	LIST		P=12F675,R=HEX,N=60
	ERRORLEVEL 	-302         	; Suppress crossing bank         
	INCLUDE		P12F675.INC	; Use Microchip's include file
	__CONFIG 	0x01C4		; INTRC,MCLR off,BOR on,WDT off	
	__IDLOCS      	0x0009      	; ID for this project
	 
	CBLOCK	0x20			; GP RAM <64 bytes>
	 TEMP_DATA,ROUND
	 D_VAR1,D_VAR2,D_VAR3
	 DURATION,DP
         REPEAT,EE_TEMP,T_CNT
         SECONDS,MINUTES,MIN_TEMP                	   
 	 PAUSE_TEMP 
 	ENDC
;************ DEFINE I/O ************
#DEFINE	SPK	    GPIO,0   		; Speaker pin  
#DEFINE HIGH_SPK     BSF  GPIO,0         
#DEFINE LOW_SPK      BCF  GPIO,0         
#DEFINE	DATA_HIGH    BSF  GPIO,1  	; HC595 Data pin
#DEFINE	DATA_LOW     BCF  GPIO,1
#DEFINE	LATCH_HIGH   BSF  GPIO,2  	; HC595 Latch pin
#DEFINE	LATCH_LOW    BCF  GPIO,2
#DEFINE	CLK_HIGH     BSF  GPIO,4  	; HC595 Clock pin
#DEFINE	CLK_LOW      BCF  GPIO,4
#DEFINE	SW_SET	    GPIO,5		; Use for setting time 
#DEFINE	SW_SRT_STP  GPIO,3		; Use for start/stop timer
;*****************************************************************
	ORG	.0000			; Reset vector	  	                  		             		                    
	GOTO    MAIN
	ORG	.0004         		; Interrupt vector
	GOTO	ISR			; Jump to interrupt routine

;***************** DEFINE TABLE ******************** 
TABLE	ADDWF	PCL,1       		; Table for 7 segment 
	DT  	0x3F,0x06,0x5B,0x4F,0x66,0x6D,0x7D,0x07,0x7F,0x6F
;	==>    	  0,   1,   2,   3,   4,   5,   6,   7,   8,   9 <== 
;*****************************************************************
MAIN	CALL	INIT_PORT		; Initial port
	CLRF	MIN_TEMP		; Clear minute temporary
	CALL	READ_EE			; Read EEPROM 
	SUBLW	"0"			; Check alarm ?
	BNZ	OLD_TIME  		; Jump if used to alarm
	MOVLW	0x78			; If first use then show t on,
	CALL	SHIFT_OUT		; 7 segment
	
;***************************************************************************
; If you press any switch before turn-on.The timer have to do nothing!
;***************************************************************************
CHK1	BTFSC	SW_SET			; Skip if pressed
	GOTO	CHK2			; Jump if not press
	MOVLW	.123			; Delay 95 mS
	CALL	DELAY2	
	GOTO	CHK1			; Wait if still pressed SW
CHK2	BTFSC	SW_SRT_STP		; Skip if pressed
	GOTO	SET_LOOP1		; Jump if not press
	MOVLW	.123			; Delay 95 mS
	CALL	DELAY2	
	GOTO	CHK1			; Wait again

;***************************************************************************
; SETUP PROGRAM Check Switch Set Press ?		
;***************************************************************************
SET_LOOP1 
	BTFSC	SW_SET			; Skip if pressed
	GOTO	$-1			; Wait for press SW
	MOVLW	.1			; Initial this variable with 0 
	MOVWF	MIN_TEMP	
	GOTO	SET_LOOP4

OLD_TIME
	CALL	READ_EE			; Read eeprom 
	MOVWF	MIN_TEMP		; Stored last time to variable 
	CALL	TABLE			; Find segment
	CALL	SHIFT_OUT		; Send data to segment
SET_NEW	BCF	T1CON,TMR1ON		; TMR1 off

SET_LOOP2
	BTFSS	SW_SET			; Skip if not pressed
	GOTO	SET_LOOP3		; Jump if press S3
	BTFSS	SW_SRT_STP		; Skip if not press
	GOTO	TIME_OK			; Jump if accepted this time
	GOTO	SET_LOOP2		; Wait for user's command

SET_LOOP3
	INCF	MIN_TEMP,1		; Increase time
	MOVLW	.10	
	SUBWF	MIN_TEMP,0		; Check variable > D'10' ?
	BZ	LOOP_9MIN		; If yes,then adjust

SET_LOOP4
	MOVFW	MIN_TEMP	
	CALL	TABLE	
	CALL	SHIFT_OUT
	CALL	SW_BEEP			; Send switch's sound
	MOVLW	.255			; Delay 200 mS
	CALL	DELAY2
	GOTO	SET_LOOP2		; Wait for use's command

LOOP_9MIN MOVLW	.1			; Adjust variable not more than,
	MOVWF	MIN_TEMP		; D'9'
	GOTO	SET_LOOP4

;***************************************************************************
; If Time OK Save Data to EEPROM
;***************************************************************************
TIME_OK	CALL	SW_BEEP
	MOVFW	MIN_TEMP
	CALL	WRITE_EE		; Stored time in EEPROM
	
;***************************************************************************
; Load Time From EEPROM
;***************************************************************************	
SET_TIME CALL	READ_EE			; Restore time to variable
	MOVWF	MIN_TEMP		
	CALL	TABLE			; Find segment
	CALL	SHIFT_OUT		; Send data to segment                      	
  	CLRF	DP			; For decimal point blinking
	COMF	DP,1		
	MOVLW	.2			; Load number of interrupt time
	MOVWF	T_CNT
	MOVLW	.60			; 1 Second
	MOVWF	SECONDS
	CLRF	MINUTES
	CALL	TMR1_SETUP		; Setting up TMR1

RELEASE1 BTFSC	SW_SRT_STP		
	GOTO	LOOP
	MOVLW	.255
	CALL	DELAY2
 	GOTO	RELEASE1
 
;***************************************************************************
; MAIN PROGRAM
;***************************************************************************	

LOOP	BTFSS	SW_SET			; Check switch S3
	GOTO	SET_NEW 		; Jump if want to set new time	  
	BTFSS	SW_SRT_STP		; Check switch S2
	CALL	PAUSE_TIME		; Jump if want to pause
	MOVFW	MINUTES			; Reload alarm value
	SUBWF	MIN_TEMP,0		; Check alarm complete ? 
	SKPZ		             	; Skip if completed
	GOTO	LOOP			; Do loop again
	
;***************************************************************************
; Routine After Alarm Completed
;***************************************************************************	
	CLRW			        ; W = 0
	CALL	TABLE	
	CALL	SHIFT_OUT		; Show 0 on 7-segment
	BCF	INTCON,GIE		; Disable all interrupt
ALM_LOOP CALL	SOUND_ALM		; Send alarm sound
	BTFSC	SW_SRT_STP		; Skip if mute
	GOTO	ALM_LOOP		; Alarm forever
	GOTO	SET_LOOP1

;***************************************************************************
; If S2 Press Timer Toggle Pause and Continue
;***************************************************************************	
	
PAUSE_TIME	
	MOVLW	B'00000001'		; Toggle TMR1ON bit
	XORWF	T1CON,1	
	CALL	SW_BEEP
	MOVFW	PAUSE_TEMP		; Restored counted time
	CALL	TABLE
	IORLW	B'10000000'		; Mask bit for show decimal point
	CALL	SHIFT_OUT
PAUSE_RLS	
	BTFSC	SW_SRT_STP		; Skip if not release yet
	RETURN			        ; Return if released
	MOVLW	.13			; Delay 10 mS
	CALL	DELAY2
	GOTO	PAUSE_RLS		; Wait for release

;***************************************************************************
;  INTERRUPT SERVICE ROUTINE 
;***************************************************************************
ISR	DECF	T_CNT,1			; Decrease variable
	BNZ	NOT_COMPLETE		; Jump if not 1 second
	MOVLW	.2			; Setup variable again
	MOVWF	T_CNT
	MOVFW	MINUTES	
	SUBWF	MIN_TEMP,0		; Substraction for invert display
	MOVWF	PAUSE_TEMP
	CALL	TABLE
	COMF	DP,1			; Complement for blinking DP
	TSTF	DP			; Check DP = ? 
	BNZ	NO_DP			; Jump if DP <> 0
SHOW_DP	IORLW	B'10000000'		; Mask bit for dp on
	GOTO	SHOW
NO_DP	IORLW	B'00000000'		; Mask bit for dp off
SHOW	CALL	SHIFT_OUT	
	DECF	SECONDS,1		; Decrease variable
	BNZ	NOT_COMPLETE		; Jump if not minute
	INCF	MINUTES,1		; Increase minute
	MOVLW	.60			; Setup variable again
	MOVWF	SECONDS			
NOT_COMPLETE	
	CALL	TMR1_SETUP	; Setup TMR1 every loop		
	RETFIE							

;----------------------------------------------------------------------
; Setup Timer 1
;----------------------------------------------------------------------
TMR1_SETUP 
 	MOVLW	B'00110000'	; PS = 1:8,Int. clk,TMR1 off 
	MOVWF   T1CON      	
	MOVLW	0x0B		; Load TMR1 <High byte>
        MOVWF   TMR1H      	
        MOVLW	0xDC		; Load TMR1 <Low  byte>
        MOVWF	TMR1L		; Therefore TMR1 = H'0BDC' = D'3036' 
        BCF     PIR1,TMR1IF	; Clear interrupt flag before enable
        BSF   	INTCON,PEIE	; Peripheral interrupt enable
        BSF	INTCON,GIE   	; Global interrupt enable                                                   
	BSF	T1CON,TMR1ON	; TMR1 on
        RETURN	

;**********************************************************************
; Subroutine Subroutine Subroutine Subroutine
;**********************************************************************

;----------------------------------------------------------------------
; Initial PORT 
;----------------------------------------------------------------------
INIT_PORT MOVLW	.7          	; Comparator off
 	MOVWF	CMCON					
	BANKSEL	ANSEL 		; //Bank 1		
	CALL	0x3FF		; Load OSC. from last address
	MOVWF	OSCCAL		; Tuning
	BSF     PIE1,TMR1IE	; TMR1 interrupt enable	
	CLRF	ANSEL		; GP4,GP2,GP1,GP0 = digital I/O
	MOVLW	B'11101000' 	; GP4,GP2,GP1,GP0 = output   
	MOVWF	TRISIO	
	BANKSEL	GPIO		; //Bank 0
	RETURN

;----------------------------------------------------------------------
; Write Data to EEPROM
;----------------------------------------------------------------------
WRITE_EE
        BANKSEL	EECON1       ; //Bank 1          
	MOVWF   EEDATA       ; Load data
        CLRF    EEADR        ; Point to address 0x00
        BSF     EECON1,WREN  ; Enable write
        MOVLW   0x55         ; Write sequence
        MOVWF   EECON2       ;
        MOVLW   0xAA         ;
        MOVWF   EECON2       ; 
        BSF     EECON1,WR    ; Start write
        BTFSC   EECON1,WR    ; Skip if write completed
        GOTO    $-1          
        BANKSEL GPIO         ; Return to bank 0
        RETURN

;----------------------------------------------------------------------
; Read Data to EEPROM
;----------------------------------------------------------------------
READ_EE
	BANKSEL EEADR        ; //Bank 1
        CLRF    EEADR        ; Point to address 0x00
        BSF     EECON1,RD    ; Start read
        MOVFW   EEDATA       ; Move data to W
        BANKSEL GPIO         ; Return to bank 0
        RETURN             

;----------------------------------------------------------------------
; Generate Sound
;----------------------------------------------------------------------
SOUND_ALM
	MOVLW   .4           ; Send beep 4 times
        MOVWF   REPEAT
RPT_LOOP MOVLW  .123   	; Duration time
        MOVWF   DURATION     
DRT_LOOP HIGH_SPK         
        CALL    SND_DELAY  	; Delay high
        LOW_SPK
        CALL    SND_DELAY   	; Delay low
        DECFSZ  DURATION,1   
        GOTO    DRT_LOOP	; Duration loop        
	MOVLW   .111       	; Delay ~85 mS
       	CALL    DELAY2
        DECFSZ  REPEAT,1     ; Beep = 4 ?        
        GOTO    RPT_LOOP     ; If no,then beep again   
        MOVLW   .3           ; Stop beep ~0.6 second
        CALL    DELAY3
        RETURN          
;----------------------------------------------------------------------
; Send Data to 74595 Shift Register
;----------------------------------------------------------------------
SHIFT_OUT
	MOVWF	TEMP_DATA	; GET DATA	
        MOVLW	.8		; 8 ROUND
        MOVWF	ROUND

LOOP595 RLF	TEMP_DATA,1	; SEND LSB FIRST
        BC	SHIFT_1
SHIFT_0 DATA_LOW	
        GOTO	CLOCK
SHIFT_1 DATA_HIGH
        GOTO    $+1		; Delay for 2 cycles
CLOCK   CLK_HIGH
        GOTO    $+1
        CLK_LOW
        GOTO    $+1
        DECFSZ	ROUND,1
        GOTO	LOOP595
        LATCH_HIGH
        GOTO    $+1
        LATCH_LOW
        GOTO    $+1
        RETURN

;----------------------------------------------------------------------
; Generate Beep
;----------------------------------------------------------------------
SW_BEEP	MOVLW	.63
	MOVWF	DURATION
SW_BEEP1 CALL	SND_DELAY
	LOW_SPK
	CALL	SND_DELAY
	DECFSZ	DURATION,1
	GOTO	SW_BEEP1
	RETURN		

;----------------------------------------------------------------------
; Generate Sound Delay
;----------------------------------------------------------------------
SND_DELAY 
	MOVLW 	.63
DELAY1  MOVWF   D_VAR1
        DECFSZ  D_VAR1,1
        GOTO    $-1
        RETURN
        
DELAY2  CLRF    D_VAR1
        MOVWF   D_VAR2
        DECFSZ  D_VAR1,1
        GOTO    $-1
        DECFSZ  D_VAR2,1 
        GOTO          $-3
	RETURN
	
DELAY3	CLRF	D_VAR1			; General purpose delay
	CLRF	D_VAR2
	MOVWF	D_VAR3
	DECFSZ	D_VAR1,1
	GOTO	$-1
	DECFSZ	D_VAR2,1
	GOTO	$-3
	DECFSZ	D_VAR3,1
	GOTO	$-5
	RETURN             
;**************** END OF PROGRAM ******************************
	ORG           0x2100       	; Point to EEPROM address 0x00
	DE            "0"          	; Specified 'NO ALARM TIME'  
	
	END
