;**********************************************************************************
; Filename :  PROJ04.ASM
; Description : Digital clock by Timer1 in PIC12F675 and display on SLCD16x2
; MCU : 	PIC12F675
; Hardware :	AX-12 PIC12F675 Activity board
; Written by : Natthapol Wongsoonthornchai
; Copyright 2003 Innovative Experiment Co.,Ltd.
;*******************************************************************************************************
  	TITLE         " DIGITAL CLOCK "

	PROCESSOR	12F675	; Specified MCU
              ERRORLEVEL 	-302         	; Suppress crossing bank         
	INCLUDE	P12F675.INC	; Use Microchip's include file
	__CONFIG	0x01C4	; INTRC,MCLR off,BOR on,WDT off	
	 
	CBLOCK	0x20		; GP RAM <64 bytes>
	 TEMP_DATA,TX_BUFFER
	 D_VAR1,D_VAR2,D_VAR3
	 COUNT,COUNT1
         REPEAT,TIME_CNT
         SECONDS,MINUTES,HOURS
         DURATION,TEMP	
	 H_BYTE,L_BYTE
	 R0,R1,R2
	 NUM_OF_CHAR,TBL_POINTER
	ENDC 
; ***********************- DEFINE I/O ****************************
#DEFINE	S3		GPIO,5	
#DEFINE	S2		GPIO,3	
#DEFINE	TX_PIN		GPIO,1	; TX pin for SLCD <RS-232>
#DEFINE	SPK		GPIO,0  ; Speaker pin  
#DEFINE HIGH_SPK     	BSF  SPK
#DEFINE LOW_SPK      	BCF  SPK
;*****************************************************************
	ORG	D'000' 		; Reset vector	  	                  		             		                    
	CALL	INITIAL
	MOVLW	.5		; Delay 1 second by approx.,
	CALL	DELAY3		; for Serial LCD start-up
	GOTO    THAI_FONT	; Write Thai font	

	ORG	D'004'        	; Interrupt vector
	GOTO	ISR		; Jump to interrupt service routine

;******************* TABLE IN BLOCK0 *************************

ASCII_TABLE	ADDWF	PCL,1	; Change BCD number to ASCII 
		DT	"0123456789"
		
TABLE_THAI	ADDWF	PCL,1	; Table for Thai language
CGRAM_ADDR00	DT   0x0A,0x15,0x11,0x11,0x15,0x15,0x1D,0x00
CGRAM_ADDR01	DT   0x00,0x00,0x19,0x0A,0x0C,0x01,0x1A,0x1C
CGRAM_ADDR02	DT   0x06,0x06,0x02,0x02,0x12,0x0A,0x06,0x00
CGRAM_ADDR03 	DT   0x04,0x04,0x04,0x04,0x04,0x06,0x06,0x00
CGRAM_ADDR04 	DT   0x0C,0x12,0x02,0x02,0x02,0x06,0x06,0x00
CGRAM_ADDR05	DT   0x0E,0x11,0x01,0x0D,0x13,0x19,0x19,0x00
CGRAM_ADDR06	DT   0x0C,0x12,0x02,0x02,0x02,0x02,0x02,0x00
CGRAM_ADDR07	DT   0x1A,0x1A,0x0A,0x12,0x12,0x17,0x1B,0x00
;******************************
; Write Thai Font To CGRAM
;******************************
THAI_FONT	MOVLW	0xFE	; Send SLCD command prefix
	CALL	TX	
	MOVLW	0x40		; Write CGRAM command 
	CALL	TX
	MOVLW	0x00
	MOVWF	TBL_POINTER	; Table address = 0
	MOVLW	D'08'*D'08'	; 8 Row and 8 Column
	MOVWF	NUM_OF_CHAR	; Write 8 characters
WRITE_CGRAM	MOVFW	TBL_POINTER
	CALL	TABLE_THAI	; Look-up table (Thai language)
	CALL	TX
	INCF	TBL_POINTER,1
	DECFSZ	NUM_OF_CHAR,1	; Skip if completed
	GOTO	WRITE_CGRAM	
;******************************
; START UP Time
;******************************
START	MOVLW	.23		; Start with 23 hour
	MOVWF	HOURS
	MOVLW	.59		; Start with 59 minute
	MOVWF	MINUTES
	MOVLW	.00		; Start with 00 second
	MOVWF	SECONDS
	MOVLW	.2		; Start with 2
	MOVWF	TIME_CNT
	CALL	TMR1_SETUP	; Initial TMR1's conditions			

;******************************
; MAIN PROGRAM
;******************************
WAIT_INT BTFSS	S3		; Wait Until Switch S3 Press
	CALL	SET_TIME
	GOTO	WAIT_INT
;******************************

;******************************
; Set New Time
;******************************
SET_TIME BCF	T1CON,TMR1ON	; TMR1 off	
	CALL	SW_BEEP
	CALL	SHOW_SET
	CALL	SHOW_MAIN
	MOVLW	0xFE		; Set line 2/14
	CALL	TX
	MOVLW	0xCD	
	CALL	TX	
	MOVLW	"E"		; Show text "ENT"
	CALL	TX
	MOVLW	"N"
	CALL	TX
	MOVLW	"T"
	CALL	TX		;	
	MOVLW	0xFE		; Turn cursor on
	CALL	TX
	MOVLW	0x0E	 
	CALL	TX		;
	MOVLW	0xFE		; Show cursor at 0xC8
	CALL	TX
	MOVLW	0xC8	
	CALL	TX		;

;******************************
; Set Cursor at Hour on LCD
;******************************
CURSOR_HR MOVLW	0xFE		; Show cursor at 0xC8
	CALL	TX
	MOVLW	0xC8	
	CALL	TX		;
	BTFSS	S3		; Skip if released S3
	GOTO	$-1		
	CALL	SW_DELAY 	; Delay for switch              	

CURSOR_HR2 BTFSS S3		; S3 press ? If no,then skip
	GOTO	CURSOR_MIN	; If yes,then set minutes 
	BTFSC	S2		; S2 press ?
	GOTO	CURSOR_HR2	; Wait for command
	INCF	HOURS,1		; HOURS = HOURS+1
	MOVLW	.24	
	SUBWF	HOURS,0		; Check HOURS = D'24' ?
	SKPNZ		        ; If no,then skip
	CLRF	HOURS		; If yes,then clear HOURS
	MOVLW	0xFE		; Show cursor at 0xC7
	CALL	TX
	MOVLW	0xC7	
	CALL	TX		;
	MOVFW	HOURS		; Load to W
	CALL	FIND_ASCII	; Show HOURS while setting
	MOVLW	"0"	
	SUBWF	R1,0		; Check ten of hour = 0 ?
	BZ	SHOW_SPC_HR	; If yes,then show space
	MOVFW	R1		; If no,then show ten of hour
	GOTO	SHOW_TEN_HR				
	
;******************************
; Show Hour
;******************************
SHOW_SPC_HR
	MOVLW	" "	
SHOW_TEN_HR
	CALL	TX
	MOVFW	R2		; Show unit of hour
	CALL	TX
	MOVLW	0xFE		; Show cursor at 0xC8
	CALL	TX
	MOVLW	0xC8		;	 
	CALL	TX
	CALL	SW_DELAY	; Delay for next loop 
	CALL	SW_DELAY
	CALL	SW_DELAY	
	CALL	SW_DELAY
	GOTO	CURSOR_HR	; Do loop again

;******************************
; Set cursor on Minute
;******************************
CURSOR_MIN
	MOVLW	0xFE	; Show cursor at 0xCB
	CALL	TX
	MOVLW	0xCB	
	CALL	TX		; 
	BTFSS	S3		; Skip if released S3
	GOTO	$-1	
	CALL	SW_DELAY 	; Delay for switch              	
CURSOR_MIN2	
	BTFSS	S3	; Skip if not press
	GOTO	CURSOR_Ent	; Goto Enter
	BTFSC	S2		; Skip if press
	GOTO	CURSOR_MIN2	; Wait for command
	INCF	MINUTES,1 	; MINUTES = MINUTES+1
	MOVLW	.60	
	SUBWF	MINUTES,0	; MINUTES = D'60' ?
	SKPNZ	              ; If no,then skip
	CLRF	MINUTES	; If yes,then clear MINUTES
	MOVLW	0xFE	; Show cursor at 0xCA
	CALL	TX
	MOVLW	0xCA	
	CALL	TX				;
	MOVFW	MINUTES	; Show MINUTES while setting
	CALL	FIND_ASCII
	MOVFW	R1	; Show ten of MINUTES
	CALL	TX
	MOVFW	R2	; Show unit of MINUTES
	CALL	TX
	MOVLW	0xFE	; Show cursor at 0xCB
	CALL	TX
	MOVLW	0xCB	
	CALL	TX
	CALL	SW_DELAY	; Delay for next loop
	CALL	SW_DELAY
	CALL	SW_DELAY
	CALL	SW_DELAY	
	GOTO	CURSOR_MIN2	; Do loop again
;******************************
; Set Cursor on ENT
;******************************
CURSOR_Ent	MOVLW	0xFE	; Show cursor at 0xCF
	CALL	TX
	MOVLW	0xCF	
	CALL	TX
	BTFSS	S3		; Skip if released
	GOTO	$-1		; Wait for release S3
	CALL	SW_DELAY 	; Delay for switch              	
CURSOR_Ent2
	BTFSS	S3		; S3 press?.If no,then skip
	GOTO	CURSOR_HR	; If yes,then set HOURS
	BTFSC	S2		; S2 press?. If yes, then ENTER
	GOTO	CURSOR_Ent2	; Do loop again
	MOVLW	0xFE		; Turn cursor off	
	CALL	TX
	MOVLW	0x0C	
	CALL	TX		;
	CLRF	SECONDS		; Clear SECONDS
	CALL	TMR1_SETUP	; Setup TMR1
	CALL	SW_BEEP
	CALL	CLR_SLCD	; Clear Serial LCD
	RETURN	              	; Return to switch loop
		
;***********************************
; INTERRUPT SERVICE ROUTINE 
;***********************************
ISR	DECFSZ	TIME_CNT,1	; Decrease variable
	GOTO	NOT_COMPLETE	; Jump if not 1 second
	MOVLW	.2		; Setup variable again
	MOVWF	TIME_CNT
		
	INCF	SECONDS,1	; SECONDS = SECONDS+1
	MOVLW	.60	
	SUBWF	SECONDS,0	; Check SECONDS = D'60' ?
	BNZ	NOT_COMPLETE  	; If no,jump to NOT_COMPLETE		
	CLRF	SECONDS		; Clear, if SECONDS = D'60'

	INCF	MINUTES,1	; MINUTES = MINUTES+1
	MOVLW	.60
	SUBWF	MINUTES,0	; Check MINUTES = D'60' ?
	BNZ	NOT_COMPLETE	; If no,jump to NOT_COMPLETE					            
	CLRF	MINUTES		; Clear, if MINUTES = D'60'

	INCF	HOURS,1		; HOURS = HOURS+1	
	MOVLW	.24	
	SUBWF	HOURS,0		; Check HOURS = D'24' ?
	BNZ	NOT_COMPLETE	; If no,jump to NOT_COMPLETE
	CLRF	HOURS		; Clear, if HOURS = D'24'
NOT_COMPLETE 
	CALL 	TMR1_SETUP	; Setup TMR1 every loop		
	CALL	SHOW_TIME	; Show time on SLCD
	
;**********************************	
; SEND 2 BEEPS EVERY HOUR 
;**********************************	
	MOVLW	.2		; Check hour cycle,if
	SUBWF	TIME_CNT,0	; TIME_CNT,SECONDS and
	BNZ	SHOW_MAIN	; MINUTES = D'00' then
	TSTF	SECONDS		; send 2 beeps to speaker
	BNZ	SHOW_MAIN
	TSTF	MINUTES	
	BNZ	SHOW_MAIN
	CALL	HR_BEEP		; Send 1st beep
	MOVLW	.130		; Delay 100 mS a beep
	CALL	DELAY2		; Send 2nd beep
	CALL	HR_BEEP

;******************************	
; Show Clock Hour
;******************************	
SHOW_MAIN MOVFW	HOURS
	CALL	FIND_ASCII
	MOVLW	"0"	
	SUBWF	R1,0		; Check ten of hour = 0 ?
	BZ	SHOW_SPACE	; If yes,then show space
	MOVFW	R1		; If no,then show ten of hour
	GOTO	SHOW_TEN				
SHOW_SPACE	MOVLW	" "	
SHOW_TEN	CALL	TX
	MOVFW	R2		; Show unit of hour
	CALL	TX
	MOVLW	":"		; Show :
	CALL	TX

;******************************	
; Show Clock Minute
;******************************		
	MOVFW	MINUTES
	CALL	FIND_ASCII
	MOVFW	R1		; Show ten of minute
	CALL	TX
	MOVFW	R2		; Show unit of minute
	CALL	TX
	MOVLW	":"		; Show :
	CALL	TX
;******************************	
; Show Clock Second
;******************************	
	MOVFW	SECONDS
	CALL	FIND_ASCII
	MOVFW	R1		; Show unit of second
	CALL	TX
	MOVFW	R2		; Show unit of second
	CALL	TX
	MOVLW	" "		; Show white space
	CALL 	TX
	MOVLW	0x07		; Show Thai unit	
	CALL	TX
	MOVLW	"."		; Show .
	CALL	TX
	RETFIE							

;***************
; Subroutines 
;***************
; Initial port and oscilator for starting 
;***************************************	
INITIAL MOVLW	.7      	; Comparator off
 	MOVWF	CMCON					
	BSF	TX_PIN		; Idle state for SLCD
	BANKSEL	ANSEL 		; Bank 1		
	CALL	0x3FF		; Load OSC. from last address
	MOVWF	OSCCAL		; Tuning
        BSF     PIE1,TMR1IE	; TMR1 interrupt enable	
	CLRF	ANSEL		; GP1,GP0 = digital I/O
	MOVLW	B'11111100' 	; GP1,GP0 = output   
	MOVWF	TRISIO
	BANKSEL	GPIO		; Bank 0
	RETURN

; Setup Timer1
;****************-		
TMR1_SETUP
  	MOVLW	B'00110000'	; PS = 1:8,Int. clk,TMR1 off 
	MOVWF   T1CON      	
	MOVLW	0x0B		; Load TMR1 <High byte>
        MOVWF   TMR1H      	
        MOVLW	0xDC		; Load TMR1 <Low  byte>
        MOVWF	TMR1L		; Therefore TMR1 = H'0BDC' = D'3036' 
	BCF     PIR1,TMR1IF	; Clear interrupt flag before enable
	BSF	T1CON,TMR1ON	; TMR1 on 
	BCF     PIR1,TMR1IF	; Clear interrupt flag before enable
        BSF	INTCON,PEIE	; Peripheral interrupt enable
        BSF     INTCON,GIE   	; Global interrupt enable                                                   
        RETURN	

; Convert DATA To ASCII
;*************************	
FIND_ASCII	CLRF	H_BYTE	
	MOVWF	L_BYTE	
	CALL	BIN2BCD		; Adjust to BCD format
	MOVFW	R1		; Find ASCII number
	CALL	ASCII_TABLE	
	MOVWF	R0		; R0 = hundred unit
	SWAPF	R2,0
	ANDLW	B'00001111'	; Mask bit
	CALL	ASCII_TABLE
	MOVWF	R1		; R1 = ten digit
	MOVFW	R2
	ANDLW	B'00001111'	; Mask bit
	CALL	ASCII_TABLE
	MOVWF	R2		; R2 = unit 
	RETURN

; SHOW SET ON LCD
;**********************	
SHOW_SET MOVLW	0xFE		; Set line 1/1
	CALL	TX
	MOVLW	0x80
	CALL	TX		;
	MOVLW	0x01		; Show char1
	CALL	TX		;
	MOVLW	0xFE		; Set line 2/1
	CALL	TX
	MOVLW	0xC0
	CALL	TX		; 
	MOVLW	0x00		; Show char0
	CALL	TX
	MOVLW	0x02		; Show char2
	CALL	TX
	MOVLW	0x03		; Show char3
	CALL	TX
	MOVLW	0x04		; Show char4
	CALL	TX
	MOVLW	0x05		; Show char5
	CALL	TX
	MOVLW	0x06		; Show char6
	CALL	TX
	MOVLW	" "		; Show white space
	CALL	TX
	RETURN

; SHOW TIME ON LCD
;**********************			
SHOW_TIME MOVLW	0xFE		; Set line 2/1
	CALL	TX
	MOVLW	0xC0
	CALL	TX		; 
	MOVLW	0x03		; Show char3
	CALL	TX
	MOVLW	0x04		; Show char4
	CALL	TX
	MOVLW	0x05		; Show char5
	CALL	TX
	MOVLW	0x06		; Show char6
	CALL	TX
	MOVLW	" "		; Show white space
	CALL	TX
	RETURN

; Send Serial Data
;**********************		
TX	MOVWF	TX_BUFFER	; 8N1,9600 BPS,NOT INVERT
	MOVLW	.8
	MOVWF	COUNT1
	CALL	DELAY_TX	; DELAY FOR IDLE STATE
	BCF	TX_PIN		; START BIT
	CALL	DELAY_TX	
LOOP_SEND RRF	TX_BUFFER,1	; SEND LSB FIRST
	BTFSC	STATUS,C
	GOTO	L2
LOGIC_0	BCF	TX_PIN		; GP5 = 0
LOGIC_1	CALL	DELAY_TX	
	DECFSZ	COUNT1,1
	GOTO	LOOP_SEND		
	BSF	TX_PIN		; STOP BIT <HIGH>
	CALL	DELAY_TX
	RETURN	
L2	BSF	TX_PIN		; GP5 = 1
	GOTO	LOGIC_1

; Geraral BEEP 
;****************	
HR_BEEP
SW_BEEP	MOVLW	.63
	MOVWF	DURATION
SW_BEEP1 HIGH_SPK
	CALL	SND_DELAY
	LOW_SPK
	CALL	SND_DELAY
	DECFSZ	DURATION,1
	GOTO	SW_BEEP1
	RETURN

; Geraral Sound
;**********************	
SND_DELAY MOVLW	.63
DELAY1  MOVWF  	D_VAR1
        DECFSZ  D_VAR1,1
        GOTO   	$-1
        RETURN

; Debound Switch Delay
;*************************		
SW_DELAY MOVLW	.66		; Delay for switch ~ 50 mS
DELAY2  CLRF  	D_VAR1
        MOVWF  	D_VAR2
        DECFSZ  D_VAR1,1
        GOTO    $-1
        DECFSZ  D_VAR2,1 
        GOTO    $-3
        RETURN
	
; Long Time Variable Delay
;*****************************		
DELAY3  CLRF    D_VAR1
       	CLRF	D_VAR2
        MOVWF   D_VAR3
        DECFSZ  D_VAR1,1
        GOTO    $-1
        DECFSZ  D_VAR2,1 
        GOTO    $-3
        DECFSZ  D_VAR3,1 
        GOTO    $-5
        RETURN

; Delay For Send Data 1 Bit
;*****************************		
DELAY_TX MOVLW	.29		; DELAY 94 uS FOR 1 BIT
	MOVWF	D_VAR1	
	DECFSZ	D_VAR1,1
	GOTO	$-1
	NOP		
	RETURN		

; Clear Serial LCD Command
;******************************		
CLR_SLCD MOVLW	0xFE		; Send SCLD command prefix
	CALL	TX
	MOVLW	0x01		; Clear display command
	CALL	TX
	MOVLW	0x00		; Delay ~200mS
	CALL	DELAY2
	RETURN  
	
; CONVERT BINARY TO BCD : Input: H_BYTE:L_BYTE Output: R0:R1:R2
;*****************************************************************************	
BIN2BCD CLRC	  	 	; Clear Carry Flag
	MOVLW   .16		; Loop 16 Times
	MOVWF   COUNT
	CLRF    R0		; Clear Output Data
	CLRF    R1
	CLRF    R2
	
LOOP16  RLF     L_BYTE,1	; Move Data 
	RLF     H_BYTE,1	; From Input To Output	
	RLF     R2,1
	RLF     R1,1
	RLF     R0,1
	DECFSZ  COUNT,1
	GOTO    ADJDEC
	RETURN
	
ADJDEC  MOVLW   R2
	MOVWF   FSR
	CALL    ADJBCD
	MOVLW   R1
	MOVWF   FSR
	CALL    ADJBCD
	MOVLW   R0
	MOVWF   FSR
	CALL    ADJBCD
	GOTO    LOOP16
	
ADJBCD  MOVLW   .3
	ADDWF   INDF,0
	MOVWF   TEMP
	BTFSC   TEMP,3          ; Test if result > 7
	MOVWF   INDF
	MOVLW   0x30
	ADDWF   INDF,0
	MOVWF   TEMP
	BTFSC   TEMP,7          ; Test if result > 7
	MOVWF   INDF            ; Save as MSD
	RETURN	

	
	END	
