;************************************************************************************
; Filename :  PROJ05.ASM
; Description : Using ADC module in PIC12F675 and display data on SLCD16x2
; MCU : 	PIC12F675
; Hardware :	AX-12 PIC12F675 Activity board
; Copyright 2003 Innovative Experiment Co.,Ltd.
;************************************************************************************
	#DEFINE	USE_SLCD
	
	LIST	P=12F675,W=-302
	INCLUDE	P12F675.INC	
	__CONFIG	0x11F4	; INTRC ,MCLR on	
	
	CBLOCK	0x20	; GP RAM <64 bytes>
	  TX_BUFFER,ROUND8,D_VAR1,D_VAR2,D_VAR3,TEMP1,TEMP2
	     ENDC
#DEFINE	SERIAL_PIN	GPIO,1	; TX/RX pin	
LF	EQU	.10	; Linefeed
CR	EQU	.13	; Carriage return

	ORG	0x000	

INIT_PORT	
MOVLW	.7	; Comparator off
	MOVWF	CMCON	
	BSF	GPIO,1	; Set GP1 latch
	BANKSEL	ANSEL	; Bank 1			
	CALL	0x3FF	; Load OSC. value
	MOVWF	OSCCAL	; Calibrate OSC.
	MOVLW	B'01111101'	; FRC <INTERNAL OSC = 500 kHZ MAX>,
	MOVWF	ANSEL	; GP1 = Digital port
	BCF	TRISIO,1	; GP1 = outpur
	BSF	INTCON,PEIE	; Peripheral enable
	BSF	PIE1,ADIE	; ADC interrupt enable
	BANKSEL	GPIO	; Return to bank 0

    IFDEF	USE_SLCD		; IF use Serial LCD
	MOVLW	.6
    	CALL	DELAY	; Delay ~1 second for SLCD set-up	
    	MOVLW	0xFE
    	CALL	TX
    	MOVLW	0x01	; Clear LCD
    	CALL	TX
    ENDIF
	
MAIN	CALL	ADC	; Main program
   
   IFDEF    USE_SLCD		; IF use Serial LCD
   	MOVLW	0xFE		; Set line1/1
   	CALL	TX
   	MOVLW	0x80
   	CALL	TX
   ENDIF
	MOVLW	"A"	; Show "ADC= "
	CALL	TX
	MOVLW	"D"
	CALL	TX
	MOVLW	"C"
	CALL	TX
	MOVLW	"="
	CALL	TX
	MOVLW	" "
	CALL	TX	

   IFDEF    USE_SLCD		; IF use Serial LCD
 	MOVLW	0xFE	; Set line2/1
	CALL	TX
	MOVLW	0xC0
 	CALL	TX
   ENDIF
  
	MOVFW	ADRESH	; Load high-byte result to W
	MOVWF	TEMP2
	CALL	SHOW	; Show high-byte result
	BANKSEL	ADRESL	; Bank 1
	MOVFW	ADRESL	; Load low-byte result to W
	MOVWF	TEMP2	
	BANKSEL	GPIO	; Return to bank 0 
	CALL	SHOW 	; Show low-byte result 

   IFNDEF	USE_SLCD		; Use computer
 	MOVLW	LF	; Send line feed
	CALL	TX
	MOVLW	CR	; Send carriage return
	CALL	TX
    ENDIF
	MOVLW	.3	; Delay 0.5 second 
	CALL	DELAY
	GOTO	MAIN	; DO again
	
ADC:	MOVLW	B'10000000'	; Right justified,Vref = Vdd,CH0
	MOVWF	ADCON0
	BSF	ADCON0,ADON	; Enable ADC module
	GOTO	$+1	; Delay >10 microsecond for Tacq
	GOTO	$+1
	GOTO	$+1
	GOTO	$+1
	GOTO	$+1	
	BSF	ADCON0,GO	; Start conversion
	NOP
	SLEEP		; Enter sleep mode
	BCF	PIR1,ADIF	; Clear ADC flag
	BCF	ADCON0,ADON	; Disable ADC
	RETURN		

SHOW:	MOVLW	.8	; Number of loop <show 0 and 1>
	MOVWF	TEMP1	
LOOP3:	RLF	TEMP2,1	; Rotate buffer
	MOVLW	'1'	; Send ASCII 1
	SKPC		; Skip if carry = 1
	MOVLW	"0"	; Send ASCII 0
	CALL	TX
	DECFSZ	TEMP1,1
	GOTO	LOOP3
	RETURN
TX:	MOVWF	TX_BUFFER	; 9600 bps,non-invert
	MOVLW	.8
	MOVWF	ROUND8
	BCF	SERIAL_PIN	; Start bit
	CALL	DELAY_TX	
TX_LOOP1:	
RRF	TX_BUFFER,1	; Send LSB first
	BTFSC	STATUS,C
	GOTO	SEND_1
SEND_0:	BCF	SERIAL_PIN	; TX pin  = 0
TX_LOOP2:	
CALL	DELAY_TX
	DECFSZ	ROUND8,1
	GOTO	TX_LOOP1
	BSF	SERIAL_PIN	; Stop bit <HIGH>
	CALL	DELAY_TX
	RETURN	
SEND_1:	BSF	SERIAL_PIN	; TX pin = 1
	GOTO	TX_LOOP2	

DELAY_TX:	
MOVLW	.29	; Delay for RS-232
	MOVWF	D_VAR1	
	DECFSZ	D_VAR1,1
	GOTO	$-1
	RETURN		
		
DELAY:	CLRF	D_VAR1	; General purpose delay
	CLRF	D_VAR2
	MOVWF	D_VAR3
	DECFSZ	D_VAR1,1
	GOTO	$-1
	DECFSZ	D_VAR2,1
	GOTO	$-3
	DECFSZ	D_VAR3,1
	GOTO	$-5
	RETURN
	
	END
