;************************************************************************************
; Filename :  PROJ06.ASM
; Description : Using ADC module in PIC12F675 as light switch
; MCU : 	PIC12F675
; Hardware :	AX-12 PIC12F675 Activity board
; Copyright 2003 Innovative Experiment Co.,Ltd.
 ;************************************************************************************
	LIST	P=12F675,W=-302
	INCLUDE	P12F675.INC
	__CONFIG	0x01F4	; INTRC(4 Mhz),WDT OFF,MCLR on

	CBLOCK	0x20	; GP RAM <64 BYTES>
	  CH0_VALUE,CH2_VALUE,SOUND_TEMP,D_VAR1
	     ENDC	

	ORG	0x000	

	MOVLW	.7	; Comparator off
	MOVWF	CMCON
	BCF	GPIO,2	; Clear GP2 latch 
	BANKSEL	OSCCAL	; Bank 1			
	CALL	0x3FF	; LOAD OSC. value FROM LAST ADDRESS
	MOVWF	OSCCAL 	; Calibrating OSC.
	BANKSEL	GPIO	; Bank 0
		
LOOP	CALL	CH0	; Find CH0 value
	CALL	CH2	; Find CH2 value
	MOVFW	CH0_VALUE	; Load CH0 result into W
	SUBWF	CH2_VALUE,0	; CH2 - CH0
	BZ	LOOP	; Jump if CH2 = CH0, sound-off
	BNC	WAKE_UP	; Jump if CH2 < CH0
	GOTO	LOOP

WAKE_UP	CALL	SOUND	; Send beep
	GOTO	LOOP
		
CH0	MOVLW	B'00000000'	; CH0, left justified
	CALL	ADC
	MOVWF	CH0_VALUE	; Store result
	RETURN 

CH2	MOVLW	B'00001000'	; CH2, left justified
	CALL	ADC
	MOVWF	CH2_VALUE	; Store result
	RETURN 

ADC	MOVWF	ADCON0	; Select channel
	BANKSEL	ANSEL	; Bank 1
	MOVLW	B'00110101'	; GP2,GP0 = Ain, FRC = 500 kHz.
	MOVWF	ANSEL	
	BANKSEL	ADCON0	; Return   to bank 0	
	BSF	ADCON0,ADON	; Enable ADC module
	GOTO	$+1	; Delay >10 microsecond for TACQ after enable ADC
	GOTO	$+1	
	GOTO	$+1
	GOTO	$+1
	GOTO	$+1
	GOTO	$+1
	GOTO	$+1	; Delau 14 microsecond
	BSF	ADCON0,GO	; Start conversion
	NOP
	BTFSC	ADCON0,GO	; Skip if completed in progress
	GOTO	$-1
	BCF	ADCON0,ADON	; Disable ADC
	MOVFW	ADRESH	; Load ADC result into W
	RETURN 			

SOUND	MOVLW	.123
	MOVWF	SOUND_TEMP
	BANKSEL	ANSEL	; Bank 1
	BCF	ANSEL,0	; GP0 = digital port
	BCF	TRISIO,0	; GP0 = output
	BANKSEL	GPIO	; Return   to bank 0
SOUND_LOOP	
BSF	GPIO,0
	MOVLW	.25
	CALL	DELAY_SOUND
	BCF	GPIO,0
	MOVLW	.111 
	CALL	DELAY_SOUND
	DECFSZ	SOUND_TEMP,1
	GOTO	SOUND_LOOP
	BANKSEL	TRISIO	; Bank 1
	BSF	TRISIO,0	; GP0 = input
	BANKSEL	GPIO	; Return   to bank 0
	RETURN 
DELAY_SOUND	
MOVWF	D_VAR1
	DECFSZ	D_VAR1,1
	GOTO	$-1
	RETURN 	
	
	END
