;***************************************************************************
; Filename :  PROJ08.ASM
; Description : Control stepper motor by PIC12F675 and PC  
; MCU : 	PIC12F675
; Hardware :	AX-12 PIC12F675 Activity board
; Written by : Natthapol Wongsoonthornchai
; Copyright 2003 Innovative Experiment Co.,Ltd.
 ;***************************************************************************
	CONSTANT 	SPEED = .100	; Set motor speed (20-255 only)
	LIST	P=12F675,W=-302
	INCLUDE	P12F675.INC	; Use Microchip include file
	__CONFIG	0x01E4	; INTRC, WDT_OFF, MCLR_OFF	

	CBLOCK	0x20	; General purpose RAM
	ROUND,TEMP_DATA,COUNT9,D_VAR1,D_VAR2,RX_BUFFER
              COUNT1,TX_BUFFER
              ENDC

#DEFINE	CLK_HIGH	BSF GPIO,4	; GP4 = clock line
#DEFINE	CLK_LOW	BCF GPIO,4
#DEFINE	LATCH_HIGH	BSF GPIO,2	; GP2 = latch line
#DEFINE	LATCH_LOW	BCF GPIO,2
#DEFINE	DATA_HIGH	BSF GPIO,1	; GP1 = data line
#DEFINE	DATA_LOW	BCF GPIO,1
#DEFINE	SERIAL_PIN	GPIO,5	; RS-232 RX pin	
;***************************************************************************
	ORG	0x000	; Reset vector
 	MOVLW	"5"
 	MOVWF	RX_BUFFER	; For STOP condition
 	GOTO	INIT_PROG	; Jump to main program 

 	ORG	0x004	; Interrupt vector
 	GOTO	ISR	; Jump to interrupt program

INIT_PROG	
	MOVLW	.7	; Comparator off
	MOVWF	CMCON		
	BANKSEL	ANSEL	; Bank 1			
	CALL	0x3FF	; Load OSC. value
	MOVWF	OSCCAL	; Tuning
	CLRF	ANSEL	; GP4:GP0 = digital I/O
	MOVLW	B'11101001'	; GP4,GP2,GP2 = output
	MOVWF	TRISIO	
INT_SETUP	
	BSF	IOCB,5	; Enable GP5 IOC 
	BSF	INTCON,GPIE	; Enable port interrupt 			
	BCF	INTCON,GPIF	; Clear IOC flag
	BSF	INTCON,GIE	; Enable all interrupt	
	BANKSEL	GPIO	; Return to bank 0	
			
STOP_MOTOR	
	CLRW		; Clear shift register
	CALL	SHIFT_OUT	
	MOVLW	"4"	; Check command
	SUBWF	RX_BUFFER,0
	BZ	TURN_LEFT
	MOVLW	"6"	; Check command
	SUBWF	RX_BUFFER,0
	BZ	TURN_RIGHT
	GOTO	STOP_MOTOR

TURN_LEFT	
	MOVLW	B'00001100'	; Left direction, Full step 2 phases
	CALL	SHIFT_OUT	
	CALL	DELAY_MOTOR
	MOVLW	"5"
	SUBWF	RX_BUFFER,0	; If command = 5 then stop motor
	BZ	STOP_MOTOR
	MOVLW	"6"
	SUBWF	RX_BUFFER,0	; If command = 6 then turn right
	BZ	TURN_RIGHT
	
	MOVLW	B'00000110'
	CALL	SHIFT_OUT
	CALL	DELAY_MOTOR
	MOVLW	"5"
	SUBWF	RX_BUFFER,0
	BZ	STOP_MOTOR
	MOVLW	"6"
	SUBWF	RX_BUFFER,0
	BZ	TURN_RIGHT

	MOVLW	B'00000011'
	CALL	SHIFT_OUT
	CALL	DELAY_MOTOR	
	MOVLW	"5"
	SUBWF	RX_BUFFER,0
	BZ	STOP_MOTOR
	MOVLW	"6"
	SUBWF	RX_BUFFER,0
	BZ	TURN_RIGHT

	MOVLW	B'00001001'
	CALL	SHIFT_OUT
	CALL	DELAY_MOTOR	
	MOVLW	"5"
	SUBWF	RX_BUFFER,0
	BZ	STOP_MOTOR
	MOVLW	"6"
	SUBWF	RX_BUFFER,0
	BZ	TURN_RIGHT
	GOTO	TURN_LEFT

TURN_RIGHT	
	MOVLW	B'00000011'	; Right direction, Full step 2 phases
	CALL	SHIFT_OUT
	CALL	DELAY_MOTOR	
	MOVLW	"5"
	SUBWF	RX_BUFFER,0	; If command = 5 then stop motor
	BZ	STOP_MOTOR
	MOVLW	"4"
	SUBWF	RX_BUFFER,0	; If command = 4 then turn left
	BZ	TURN_LEFT
	
	MOVLW	B'00000110'
	CALL	SHIFT_OUT
	CALL	DELAY_MOTOR	
	MOVLW	"5"
	SUBWF	RX_BUFFER,0
	BZ	STOP_MOTOR
	MOVLW	"4"
	SUBWF	RX_BUFFER,0
	BZ	TURN_LEFT

	MOVLW	B'00001100'
	CALL	SHIFT_OUT
	CALL	DELAY_MOTOR	
	MOVLW	"5"
	SUBWF	RX_BUFFER,0
	BZ	STOP_MOTOR
	MOVLW	"4"
	SUBWF	RX_BUFFER,0
	BZ	TURN_LEFT

	MOVLW	B'00001001'
	CALL	SHIFT_OUT
	CALL	DELAY_MOTOR	
	MOVLW	"5"
	SUBWF	RX_BUFFER,0
	BZ	STOP_MOTOR
	MOVLW	"4"
	SUBWF	RX_BUFFER,0
	BZ	TURN_LEFT
	GOTO	TURN_RIGHT

;***************************************************************************
; INTERRUPT SERVICE ROUTINE 
;***************************************************************************
ISR	MOVLW	.9	; Find command from computer
	MOVWF	COUNT9
	CALL	DELAY_HALF	; Delay 1/2 bit	
RX_LOOP2	
	CALL	DELAY_BIT	; Delay 1 bit 
	CLRC
	BTFSC	SERIAL_PIN
	SETC
	RRF	RX_BUFFER,1
	DECFSZ	COUNT9,1
	GOTO	RX_LOOP2
	RLF	RX_BUFFER,1	; Carry = stop bit
	BTFSS	SERIAL_PIN	; Wait for rising edge <STOP BIT>
	GOTO	$-1
	MOVFW	RX_BUFFER
	BCF	INTCON,GPIF	; Clear interrupt flag	
	RETFIE		; Return from interrupt
;***************************************************************************
;SUBROUTINES  
;***************************************************************************
SHIFT_OUT	
	MOVWF	TEMP_DATA	; Get data	
	MOVLW	.8	; 8 rounds
	MOVWF	ROUND
LOOP595	RLF	TEMP_DATA,1	; Send LSB first
	BC	SHIFT_1
SHIFT_0	DATA_LOW	
	GOTO	CLOCK
SHIFT_1	DATA_HIGH
	NOP
CLOCK	CLK_HIGH
	NOP
	CLK_LOW
	NOP
	DECFSZ	ROUND,1
	GOTO	LOOP595
	LATCH_HIGH
	NOP
	LATCH_LOW
	NOP
	RETURN	

DELAY_BIT	
	MOVLW	.29	; Delay 1 bit 
	MOVWF	D_VAR1	
	DECFSZ	D_VAR1,1
	GOTO	$-1
	RETURN		
	
DELAY_HALF	
	MOVLW	.16	; Delay 1/2 bit	
	MOVWF	D_VAR1	
	DECFSZ	D_VAR1,1
	GOTO	$-1	
	RETURN		
	
DELAY_MOTOR	
	MOVLW	SPEED	; Load delay value 
	MOVWF	D_VAR2	
	CLRF	D_VAR1
	DECFSZ	D_VAR1,1
	GOTO	$-1
	DECFSZ	D_VAR2,1
	GOTO	$-3
	RETURN

	END

