;*************************************************************************************
; Filename :  PROJ09.ASM
; Description : Read temperature from DS1820 with PIC12F675 and show on SLCD16x2
; MCU : 	PIC12F675
; Hardware :	AX-12 PIC12F675 Activity board
; Written by : Natthapol Wongsoonthornchai
; Copyright 2003 Innovative Experiment Co.,Ltd.
;*************************************************************************************

;*************************
; ASSIGN OUTPUT MONITOR 
;*************************
    #DEFINE	USE_PC	; USE this line if want to show on computer
   ; #DEFINE	USE_SLCD	; Use this line if want to show on SLCD

;*************************************************************************************
 	LIST	P=12F675,W=-302
	INCLUDE	P12F675.INC	; Use MICROCHIP's include file
	__CONFIG	0x01E4	; INTRC, MCLR ON, BODEN_ON, PWRTEN_ON	

;*************************
; ASSIGN DATA RAM
;*************************
    	CBLOCK	0x20	; RAM
	TEMP_WR,TEMP_RD,COUNT,COUNT1,COUNT8,D_VAR1,D_VAR2,D_VAR3
	TEMP_DATA,TEMP,H_BYTE,L_BYTE,R0,R1,R2,TX_BUFFER,TEMP_MSB
	TEMP_LSB,SIGN,DP	
	ENDC 

;*************************
; ASSIGN PORT 
;*************************
#DEFINE	DQ2_PIN	GPIO,2	; DS1820 DQ pin
#DEFINE	TX_PIN	GPIO,1	; RS-232 pin


	ORG	0x000

	MOVLW	.7	; Iitial port
	MOVWF	CMCON	; Turn-off comparator	
	BSF	TX_PIN	; Set GP1 for TX idle state
	BANKSEL	ANSEL	; !!Bank 1			
	BCF	TRISIO,1	; GP1 = output <RS-232>
	BCF	TRISIO,0	; GP0 = output <Piezo>
	CALL	0x3FF	; LOAD OSC. VALUE
	MOVWF	OSCCAL	; Calibratibg here
	CLRF	ANSEL	; GP4,GP2:GP0 = digital port
	BANKSEL	GPIO	; !!Bank0
	CALL	READ_TEMP	; Removed DS1820'S first result<D'85'>
	MOVLW	.5
	CALL	DELAY	; Delay 1 sec for SLCD start-up

    IFDEF	USE_SLCD	
	CALL	CLR_SLCD	; Clear SLCD'S screen
    ENDIF	
;*************************************************************************************
; MAIN PROGRAM 
;*************************************************************************************

LOOP	CALL	READ_TEMP	; Read TL and TH   
	CALL	TEMP_CAL	; Calculate temperature
	CALL	SHOW_TEMP	; Show on SLCD
	MOVLW	.5	; Delay 0.5 sec before read next temp
	CALL	DELAY	
	GOTO	LOOP
;*************************
; CALCULATING TEMPERATURE 
;*************************
TEMP_CAL	
MOVLW	" " 	; Show white space for positive temp.
	MOVWF	SIGN		
	TSTF	TEMP_MSB	; Test file, pos or neh temp.?
	BZ	POS_TEMP	; Branch (jump) if positive temp.
NEG_TEMP	
MOVLW	"-"
	MOVWF	SIGN	; Show minus sign for negative temp.
	NEGF	TEMP_LSB,1	; Change to 2's complement
POS_TEMP	
CLRC		; Clear carry flag before rotate	
	RRF	TEMP_LSB,1	; Temperature = TEMP_LSB / 2
	MOVLW	"0"
	MOVWF	DP	; For show 0.0 degree
	MOVLW	"5"	; For show 0.5 degree
	SKPNC		; Skip if carry=0 
	MOVWF	DP	
	CLRF	H_BYTE	
	MOVFW	TEMP_LSB
	MOVWF	L_BYTE
	CALL	BIN2BCD	; Adjust result to BCD format
FIND_NUMBER	
MOVFW	R1	; Finding ASCII number
	CALL	ASCII_TABLE	
	MOVWF	R0	; R0 = hundred unit
	SWAPF	R2,0
	ANDLW	B'00001111'	; Mask bit
	CALL	ASCII_TABLE
	MOVWF	R1	; R1 = ten digit
	MOVFW	R2
	ANDLW	B'00001111'	; Mask bit
	CALL	ASCII_TABLE
	MOVWF	R2	; R2 = unit 
	RETURN
;*************************
; Change result TO ASCII 
;*************************
ASCII_TABLE	ADDWF	PCL,1	; Change BCD to ASCII number
	DT	"0123456789"
;***************************************************************************
SHOW_TEMP	

   IFDEF	USE_SLCD		; Show ON SLCD
	MOVLW	0xFE	; Set SLCD line 1/1
	CALL	TX
	MOVLW	0x80	
	CALL	TX
   ENDIF	
	MOVLW	"T"	; Show Temp on SLCD line 1/1
	CALL	TX	
	MOVLW	"e"
	CALL	TX
	MOVLW	"m"
	CALL	TX
	MOVLW	"p"
	CALL	TX
	MOVLW	"="
	CALL	TX
	MOVFW	SIGN	; Sign
	CALL	TX	
	MOVFW	R0	; Hundred
	CALL	TX
	MOVFW	R1	; Ten
	CALL	TX
	MOVFW	R2	; Unit
	CALL	TX
	MOVLW	"."	; .
	CALL	TX
	MOVFW	DP	; Decimal point
	CALL	TX	
	MOVLW	" "	; White space
	CALL	TX
	MOVLW	"C"	; Degree
	CALL	TX
  
   IFDEF	USE_PC		; Assign constant for PC's terminal program	

   	CONSTANT	LF = .10	; Send Line Feed
	CONSTANT	CR = .13	; Send Carriage Return 
	
	MOVLW	CR	
	CALL	TX
	MOVLW	LF
	CALL	TX
   ENDIF	
	RETURN

;*************************
; 1-WIRE SUBROUTINES 
;*************************
READ_TEMP	
CALL	RESET_PULSE	; Master send reset pulse
	CALL	PRES_PULSE	; Master Receive prescense pulse <ACK.>
	MOVLW	0xCC	; Send skip rom command
	CALL	WRITE
	MOVLW	0x44	; Send convert temp command
	CALL	WRITE
	CALL	RESET_PULSE	; Master send reset pulse before read
	CALL	PRES_PULSE	; Master Receive prescense pulse <ACK.>
	MOVLW	0xCC	; Send skip rom command
	CALL	WRITE
	MOVLW	0xBE	; Send read temp command
	CALL	WRITE	
	CALL	READ	; First read. Read TL
	MOVWF	TEMP_LSB	; Stored TL in w
	CALL	READ	; Second read. Read TH
	MOVWF	TEMP_MSB	; Store TH in w
	RETURN
READ	MOVLW	.8	; Set number of loop
	MOVWF	COUNT8
READ_LOOP2	
BCF	DQ2_PIN	; Clear output latch
	BSF	STATUS,RP0	; !!Bank 1
	BCF	DQ2_PIN	; DQ = Output
	BCF	STATUS,RP0	; !!BANK 0
	GOTO	$+1
	BSF	STATUS,RP0	; !!BANK 1
	BSF	DQ2_PIN	; Set to input port
	BCF	STATUS,RP0	; !!Bank 0
	BTFSC	DQ2_PIN	; Read data at this pin
	GOTO	READ_1_2
READ_0_2	
CLRC		; Clear carry flag
READ_L2_2	
RRF	TEMP_RD,1	; Rotate right for receive LSB first	
	CALL	DELAY45U
	DECFSZ	COUNT8,1	; Skip if completed
	GOTO	READ_LOOP2	; Do loop again 
	MOVFW	TEMP_RD	; w = Result
	RETURN
READ_1_2	
SETC		; Set carry flag
	GOTO	READ_L2_2	

RESET_PULSE	
BCF	DQ2_PIN	; DQ = low
	BSF	STATUS,RP0	; !!Bank 1
	BCF	DQ2_PIN	; Set to output port
	BCF	STATUS,RP0	; !!Bank 0
	CALL	DELAY617U	; For reset pulse
	BSF	STATUS,RP0	; !!Bank 1
	BSF	DQ2_PIN	; DQ = Hi Z.
	BCF	STATUS,RP0 	; !!Bank 0 
	RETURN
PRES_PULSE    
BSF	STATUS,RP0	; !!Bank 1
	BSF	DQ2_PIN	; DQ = Hi Z.
	BCF	STATUS,RP0	; !!Bank 0 
	CALL	DELAY15U
	BTFSC	DQ2_PIN	; Skip if clear
	GOTO	$-1	; Wait for presence pulse
	BTFSS	DQ2_PIN	; Skip if data line = float
	GOTO	$-1	; Wait for float
	RETURN
WRITE	MOVWF	TEMP_WR
	MOVLW	.8	; Number of loop
	MOVWF	COUNT8
WR_LOOP_2	
RRF	TEMP_WR,1
	BC	SEND_1_2	; Branch if C = 1
SEND_0_2	
BCF	DQ2_PIN	; Clear output latch
	BSF	STATUS,RP0	; !!Bank 1
	BCF	DQ2_PIN	; Set to output port
	BCF	STATUS,RP0	; !!Bank 0
	CALL	DELAY45U
L3_2	CALL	DELAY15U	
	BSF	STATUS,RP0	; !!Bank 1
	BSF	DQ2_PIN	; DQ = float
	BCF	STATUS,RP0	; !!Bank 0
	NOP		; Delay 1uS
	DECFSZ	COUNT8,1	; Skip if completed
	GOTO	WR_LOOP_2	; Do loop again
	RETURN
SEND_1_2	
BCF	DQ2_PIN	; Clear output latch
	BSF	STATUS,RP0	; !!Bank 1
	BCF	DQ2_PIN	; DQ = output
	BCF	STATUS,RP0	; !!Bank 0
	GOTO	L3_2
	
;****************************
; SEND TO SERIAL OUT (PC or SLCD)
 ;***************************
TX	MOVWF	TX_BUFFER	; Send to RS-232 (PC or SLCD)
	MOVLW	.8	; 8N1,9600 BPS, not invert
	MOVWF	COUNT1
	CALL	DELAY_TX	; Delay for idle state
	BCF	TX_PIN	; Send start bit <logic 0>
	CALL	DELAY_TX	
LOOP_SEND	
RRF	TX_BUFFER,1	; Send LSB first
	BTFSC	STATUS,C
	GOTO	L2
LOGIC_0	BCF	TX_PIN	; Send logic 0
LOGIC_1	CALL	DELAY_TX	
	DECFSZ	COUNT1,1
	GOTO	LOOP_SEND		
	BSF	TX_PIN	; Send stop bit<logic 1>
	CALL	DELAY_TX
	RETURN	
L2	BSF	TX_PIN	; Send logic 1	
	GOTO	LOGIC_1
	
;*************************
; DELAY SUBROUTINES 
;*************************
DELAY15U	
MOVLW	.5	; Delay 15 uS
	MOVWF	D_VAR1
	DECFSZ	D_VAR1,1
	GOTO	$-1
	RETURN 
DELAY45U	
MOVLW	.15           ; Delay 45 uS
	MOVWF	D_VAR1
	DECFSZ	D_VAR1,1
	GOTO	$-1
	NOP
	RETURN	
DELAY617U	
MOVLW	.206	; Delay 617 uS
	MOVWF	D_VAR1
	DECFSZ	D_VAR1,1
	GOTO	$-1
	RETURN
DELAY_TX	
MOVLW	.29	; Delay for RS-232
	MOVWF	D_VAR1	
	DECFSZ	D_VAR1,1
	GOTO	$-1
	RETURN		
DELAY	CLRF	D_VAR1	; Genaral purpose delay
	CLRF	D_VAR2
	MOVWF	D_VAR3	; D_VAR3 = 1-255
	DECFSZ	D_VAR1,1
	GOTO	$-1
	DECFSZ	D_VAR2,1
	GOTO	$-3
	DECFSZ	D_VAR3,1
	GOTO	$-5
	RETURN

;************************* 
; CONVERT BINARY TO BCD 
; Input  : H_BYTE:L_BYTE
; Output : R0:R1:R2
;*************************
BIN2BCD  	
CLRC	  	 
	MOVLW   .16
	MOVWF   COUNT
	CLRF    R0
	CLRF    R1
	CLRF    R2
LOOP16  RLF     L_BYTE,1
	RLF     H_BYTE,1
	RLF     R2,1
	RLF     R1,1
	RLF     R0,1
	DECFSZ  COUNT,1
	GOTO    ADJDEC
	RETURN
ADJDEC 	MOVLW   R2
	MOVWF   FSR
	CALL    ADJBCD
	MOVLW   R1
	MOVWF   FSR
	CALL    ADJBCD
	MOVLW   R0
	MOVWF   FSR
	CALL    ADJBCD
	GOTO    LOOP16
ADJBCD 	MOVLW   .3
	ADDWF   0,0
	MOVWF   TEMP
	BTFSC   TEMP,3          ; Test if result > 7
	MOVWF   0
	MOVLW   30
	ADDWF   0,0
	MOVWF   TEMP
	BTFSC   TEMP,7          ; Test if result > 7
	MOVWF   0               ; Save as MSD
	RETURN
;*********************************
; SHORT PROGRAM FOR SLCD16x2 ONLY 
;*********************************
CLR_SLCD	
MOVLW	0xFE	; Send SCLD command prefix
	CALL	TX
	MOVLW	0x01	; Clear display command
	CALL	TX
	MOVLW	.1	; Delay ~200mS
	CALL	DELAY
	RETURN  

	END

