;*************************************************************************************
; Filename :  PROJ10.ASM
; Description : 2-language thermometer 
; MCU : 	PIC12F675
; Hardware :	AX-12 PIC12F675 Activity board
; Written by : Natthapol Wongsoonthornchai
; Copyright 2003 Innovative Experiment Co.,Ltd.
;*************************************************************************************
 	LIST	P=12F675	; Specified CPU
 	ERRORLEVEL	-302	; Suppressed crossing bank warning
	INCLUDE	P12F675.INC	; Use MICROCHIP's include file

	__CONFIG	0x01E4	; INTRC, MCLR ON, BODEN_ON, PWRTEN_ON
	__IDLOCS	0x1820 	; ID of this project
	TITLE	"MEASURING TEMPERATURE IN 2 UNIT"
	SUBTITLE 	"SHOW IN THAI & ENGLISH LANGUAGE"

	ORG	0x2100
	DE	"INEX"	; INEX'S product

    	CBLOCK	0x20	; Our RAM
	TEMP_WR,TEMP_RD,COUNT,COUNT1,COUNT8,ROUND,D_VAR1,D_VAR2,D_VAR3
	TEMP_DATA,ROUND2,TEMP,H_BYTE,L_BYTE,R0,R1,R2,TX_BUFFER,BEEP_TEMP
	BIN8,TEMP2,T_LSB,COUNTER,MULTIPLICAND,MULTIPLIER,PRODUCT_H,PRODUCT_L		
	QUOTIENT_HIGH,QUOTIENT_LOW,R1H,R1L,NUM_OF_CHAR,TBL_POINTER
	R1HIGH,R1LOW,REMAINDER_HIGH,REMAINDER_LOW,CELCIUS1,CELCIUS2,POINT
	TEMP_LAN,TEMP_UNIT,TIME_SW
	ENDC 

#DEFINE	DQ2_PIN	GPIO,2	; DS1820 DQ pin
#DEFINE	TX_PIN	GPIO,1	; RS-232 pin
#DEFINE	SW3	GPIO,5	; For select unit <degree in C or F>
#DEFINE	PIEZO	GPIO,0	; Beep out
;*************************************************************************************

	ORG	0x000
RESET	GOTO	INIT_PORT	

;-----------------------  ASSIGNED ALL TABLE IN BLOCK0  -----------------------------
ASCII_TABLE	ADDWF	PCL,1	; Convert BCD to ASCII
	DT	"0123456789"
TABLE_THAI	ADDWF	PCL,1	; Table for THAI language
CGRAM_ADDR00 	DT   0x0E,0x11,0x01,0x19,0x11,0x0E,0x03,0x01 ;  
CGRAM_ADDR01 	DT   0x0D,0x15,0x15,0x15,0x17,0x19,0x00,0x00 ; 
CGRAM_ADDR02	DT   0x1B,0x1B,0x0A,0x0D,0x09,0x09,0x00,0x00 ; 
CGRAM_ADDR03	DT   0x0E,0x11,0x09,0x09,0x19,0x19,0x05,0x07 ; 
CGRAM_ADDR04	DT   0x19,0x19,0x09,0x09,0x1F,0x19,0x00,0x00 ; 
CGRAM_ADDR05	DT   0x00,0x00,0x00,0x00,0x00,0x00,0x0E,0x1F ;  
CGRAM_ADDR06	DT   0x01,0x02,0x15,0x0D,0x05,0x05,0x02,0x00 ; 
CGRAM_ADDR07  DT   0x01,0x01,0x19,0x09,0x15,0x15,0x1B,0x00 ; 
;------------------------------------------------------------------------------------    	

INIT_PORT	MOVLW	.7	; Turn-off comparator
	MOVWF	CMCON		
	BSF	TX_PIN	; Set GP1 for TX idle state
	BANKSEL	ANSEL	; !!Bank 1			
	BCF	TRISIO,1	; GP1 = output <RS-232>
	BCF	TRISIO,0	; GP0 = output <Piezo>
	CALL	0x3FF	; LOAD OSC. VALUE
	MOVWF	OSCCAL	; Calibratibg here
	CLRF	ANSEL	; GP4,GP2:GP0 = digital port
	BANKSEL	GPIO	; !!Bank0
	CALL	READ_TEMP	; Removed DS1820'S first result<D'85'>
	MOVLW	.5
	CALL	DELAY	; Delay 1 sec for SLCD start-up
	CALL	CLR_SLCD
WRITE_THAI	MOVLW	0xFE	; Send SLCD command prefix
	CALL	TX	
	MOVLW	0x40	; Write CGRAM command (write THAI letters) 
	CALL	TX
	MOVLW	0x00
	MOVWF	TBL_POINTER	; Table address = 0
	MOVLW	0x08*0x08
	MOVWF	NUM_OF_CHAR	; Write bitmap 64 letters
WRITE_CGRAM1	MOVFW	TBL_POINTER
	CALL	TABLE_THAI	; Look-up table (for THAI letters)
	CALL	TX
	INCF	TBL_POINTER,1
	DECFSZ	NUM_OF_CHAR,1	; Skip if completed
	GOTO	WRITE_CGRAM1  
; ************************************************************************
; MAIN
; ************************************************************************
	BANKSEL	PCON	; !!Bank 1
	BTFSC	PCON,1	; Skip if POR. occured
	GOTO	CHK_UNIT	; Jump if reset by MCLR pin 			
	BSF	PCON,1	; Set bit POR after AFTER POR. occured
	MOVLW	"C"
	MOVWF	TEMP_UNIT
	BANKSEL	GPIO	; !!Bank 0
	GOTO	THAI_MAIN
CHK_UNIT	MOVLW	"T"	
	BANKSEL	GPIO	; !!Bank 0
	SUBWF	TEMP_LAN,0	; Temperature unit = ?
	BZ	ENG_MAIN	; Jump to ENGLISH unit
	MOVLW	"E"
	SUBWF	TEMP_LAN,0
	BZ	THAI_MAIN	; Jump to THAI unit
	B	RESET 	; Goto reset if unit failed(B=BRANCH=GOTO) 		
;**************************
; FOR THAI LANGUAGE ROUTINE 
;**************************
THAI_MAIN	
MOVLW	"C"
	SUBWF	TEMP_UNIT,0
	BZ	THAI_IN_C
	B	THAI_IN_F
THAI_IN_C	
BTFSS	SW3	; Wait sw released
	GOTO	$-1
	MOVLW	"T"	; Specified THAI unit
	MOVWF	TEMP_LAN
	CALL	READ_TEMP
	MOVWF	CELCIUS1	; Result for Celcius cal.
	CALL	T_CELCIUS	; Convert temp in C unit
	CALL	SHOW_THAI_C	
	MOVLW	"C"	; Specified unit
	MOVWF	TEMP_UNIT	
	MOVLW	.50	; Delay loop = 50*10mS = 500 mS
	MOVWF	TIME_SW
SW_LOOP1	CALL	DELAY_SW	
	BTFSS	SW3	; Skip if not press
	GOTO	SW_LOOP2	
	DECFSZ	TIME_SW,1	; Skip if delay completed
	GOTO	SW_LOOP1
	GOTO	THAI_IN_C	; Do this job again
SW_LOOP2	CALL	BEEP	; Beep out
	GOTO	THAI_IN_F

THAI_IN_F	BTFSS	SW3	; Wait sw released
	GOTO	$-1
	MOVLW	"T"	; Specified THAI unit
	MOVWF	TEMP_LAN
	CALL	READ_TEMP
	MOVWF	CELCIUS2	; Result for Fahrenheit cal.
	CALL	T_FAHRENHEIT	; Convert temp in F unit
	CALL	SHOW_THAI_F	
	MOVLW	"F"
	MOVWF	TEMP_UNIT
	MOVLW	.50
	MOVWF	TIME_SW
SW_LOOP3	CALL	DELAY_SW	
	BTFSS	SW3	
	GOTO	SW_LOOP4
	DECFSZ	TIME_SW,1
	GOTO	SW_LOOP3
	GOTO	THAI_IN_F
SW_LOOP4	CALL	BEEP
	GOTO	THAI_IN_C

;************************  
; FOR ENGLISH LANGUAGE ROUTINE
;*************************
ENG_MAIN	MOVLW	"C"	
	SUBWF	TEMP_UNIT,0	; Check unit = C or F ?
	BZ	ENG_IN_C	; Jump if unit in C
	B	ENG_IN_F	; Jump if unit in F
ENG_IN_C	BTFSS	SW3	; Skip if not press sw3
	GOTO	$-1	; Wait for sw released
	MOVLW	"E"	; Specified ENGLISH unit
	MOVWF	TEMP_LAN
	CALL	READ_TEMP
	MOVWF	CELCIUS1	; Result for Celcius cal.
	CALL	T_CELCIUS	; Convert temp in C unit
	CALL	SHOW_CELCIUS	
	MOVLW	"C"
	MOVWF	TEMP_UNIT
	MOVLW	.50
	MOVWF	TIME_SW
SW_LOOP5	CALL	DELAY_SW	
	BTFSS	SW3	
	GOTO	SW_LOOP6
	DECFSZ	TIME_SW,1
	GOTO	SW_LOOP5
	GOTO	ENG_IN_C
SW_LOOP6	CALL	BEEP
	GOTO	ENG_IN_F
	
ENG_IN_F	BTFSS	SW3	; Wait for sw released
	GOTO	$-1
	MOVLW	"E"	; Specified ENGLISH unit
	MOVWF	TEMP_LAN
	CALL	READ_TEMP
	MOVWF	CELCIUS2	; Result for Fahrenheit cal.
	CALL	T_FAHRENHEIT	; Convert temp in F unit
	CALL	SHOW_FAHRENHEIT
	MOVLW	"F"
	MOVWF	TEMP_UNIT
	MOVLW	.50
	MOVWF	TIME_SW
SW_LOOP7	CALL	DELAY_SW	
	BTFSS	SW3	
	GOTO	SW_LOOP8
	DECFSZ	TIME_SW,1
	GOTO	SW_LOOP7
	GOTO	ENG_IN_F
SW_LOOP8	CALL	BEEP
	GOTO	ENG_IN_C

SHOW_CELCIUS  CALL	ENG_MESS1	; Show temp in Celcius
	CALL	SHOW
	MOVLW	0xDF	; Send degree symbol
 	CALL	TX
	MOVLW	"C"	; Send C unit
	CALL	TX	
	RETURN
SHOW_FAHRENHEIT		; Show temp in Fahrenheit
	CALL	ENG_MESS1
	CALL	SHOW
	MOVLW	0xDF	; Send degree symbol
	CALL	TX	
	MOVLW	"F"	; Send F unit
	CALL	TX	
	RETURN
ENG_MESS1	MOVLW	0xFE
	CALL	TX
	MOVLW	0xC0
	CALL	TX
	MOVLW	'T'	; Shoe temperature
	CALL	TX
	MOVLW	'e'	
	CALL	TX
	MOVLW	'm'	
	CALL	TX
	MOVLW	'p'
	CALL	TX
	MOVLW	'='	
	CALL	TX
	RETURN	
T_CELCIUS	MOVWF	CELCIUS1
	CLRC		
	RRF	CELCIUS1,1	; TEMP_RD/2
	MOVLW	'0'	; Show .0 degree
	MOVWF	POINT
	BNC	CONVERT1	; Jump if carry = 0
	MOVLW	'5'	; Show .5 degree
	MOVWF	POINT
CONVERT1	CLRF	H_BYTE
	MOVFW	CELCIUS1
	MOVWF	L_BYTE
	CALL	BIN2BCD	; Convert binary to BCD
FIND_NUMBER	MOVFW	R1	
	CALL	ASCII_TABLE	; Find ASCII number
	MOVWF	R0	; R0 = hundred unit
	ADDLW	-0x30	; w = w-0x30 ('0')
	BNZ	CONVERT2	; Jump if w <> 0
	MOVLW	' '	; Show white space if temp <100 C
	MOVWF	R0
	SWAPF	R2,0
	ANDLW	B'00001111'	; Mask bit
	CALL	ASCII_TABLE
	MOVWF	R1	; R1 = ten digit
	ADDLW	-0x30	; w = w-0x30 ('0')
	BNZ	CONVERT3	; Jump if w <> 0
	MOVLW	' '	; Show white spaceHOW if temp < 10 C
	MOVWF	R1	
	GOTO	CONVERT3
CONVERT2	SWAPF	R2,0
	ANDLW	B'00001111'	; Mask bit
	CALL	ASCII_TABLE
	MOVWF	R1	; R1 = ten digit

CONVERT3	MOVFW	R2
	ANDLW	B'00001111'	; Mask bit
	CALL	ASCII_TABLE
	MOVWF	R2	; R2 = unit degree
	RETURN
T_FAHRENHEIT	CLRC
	RRF	CELCIUS2,1	; Divide by 2
	MOVLW	.9	; .5 C = .9 F	
	MOVWF	POINT	
	SKPC		; Skip if dp
	CLRF	POINT	; .0 degree
	MOVFW	CELCIUS2
	MOVWF	MULTIPLICAND
	CALL	MUL8
	MOVFW	PRODUCT_H
	MOVWF	QUOTIENT_HIGH
	MOVFW	PRODUCT_L
	MOVWF	QUOTIENT_LOW
	CALL	DIV16
	MOVFW	POINT	; load .9 F					
	ADDWF	REMAINDER_LOW,0
	MOVWF	L_BYTE
	CLRF	H_BYTE
	CALL	BIN2BCD
	MOVFW	R2	
	MOVWF	POINT	; POINT = dp result
	BTFSS	POINT,4	; Adjust dp. for Fahrenheit
	GOTO	NO_ADJ	
	INCF	QUOTIENT_LOW,1; Adjust Fahrenheit unit
	SKPNZ
	INCF	QUOTIENT_HIGH,1; Increase if QUOTIENT_LOW > D'255'
NO_ADJ	MOVLW	.32	; Load constant
	ADDWF	QUOTIENT_LOW,1
	SKPNC
	INCF	QUOTIENT_HIGH,1; Increase if carry out
	BCF	POINT,4	; Truncateed bit4
	MOVFW	POINT
	CALL	ASCII_TABLE 
	MOVWF	POINT 
	MOVFW	QUOTIENT_HIGH
	MOVWF	H_BYTE
	MOVFW	QUOTIENT_LOW
	MOVWF	L_BYTE
	CALL	BIN2BCD
	CALL	FIND_NUMBER
	RETURN
		
SHOW	MOVFW	R0	; Show hundred
	CALL	TX
	MOVFW	R1	; Show ten
	CALL	TX
	MOVFW	R2	; Show unit
	CALL	TX
	MOVLW	'.'	; Show .
	CALL	TX
	MOVFW	POINT	; Show decimal
	CALL	TX
	RETURN	

SHOW_THAI_C	CALL	SHOW_THAI
	MOVLW	0x06	; Write 
	CALL	TX
	RETURN
SHOW_THAI_F	CALL	SHOW_THAI
	MOVLW	0x07	; Write 
	CALL	TX
	RETURN
SHOW_THAI	MOVLW	0xFE	
	CALL	TX
	MOVLW	0x84	; DDRAM addr. 0x84 (line1/5)
	CALL	TX	; Write 
	MOVLW	5
	CALL	TX
	MOVLW	0xFE
	CALL	TX
	MOVLW	0xC0	; DDRAM addr. 0xC0 (line2/1)
	CALL	TX
	MOVLW	0x00
	CALL	TX
	MOVLW	0x01
	CALL	TX
	MOVLW	0x02 
	CALL	TX
	MOVLW	0x03
	CALL	TX
	MOVLW	0x04
	CALL	TX
	MOVLW	"="
	CALL	TX
	CALL	SHOW
	MOVLW	0xDF	; For HD44780 or compatible only <degree symbol>
	CALL	TX
	RETURN
;***************************
; 1-WIRE ROUTINE 
;***************************
READ_TEMP	CALL	RESET_PULSE
	CALL	PRES_PULSE
	MOVLW	0xCC	; Send skip rom command
	CALL	WRITE
	MOVLW	0x44	; Send convert temp command
	CALL	WRITE
	CALL	RESET_PULSE
	CALL	PRES_PULSE
	MOVLW	0xCC	; Send skip rom command
	CALL	WRITE
	MOVLW	0xBE	; Send read temp command
	CALL	WRITE	
	CALL	READ
	RETURN
READ	MOVLW	.8
	MOVWF	COUNT8
READ_LOOP2	BCF	DQ2_PIN
	BSF	STATUS,RP0	; !!Bank 1
	BCF	DQ2_PIN	; DQ = OUTPUT
	BCF	STATUS,RP0	; !!BANK 0
	NOP
	NOP
	BSF	STATUS,RP0	; !!BANK 1
	BSF	DQ2_PIN	; Set to input port
	BCF	STATUS,RP0	; !!Bank 0
	BTFSC	DQ2_PIN	; Read data at this pin
	GOTO	READ_1_2
READ_0_2	CLRC
READ_L2_2	RRF	TEMP_RD,1	
	CALL	DELAY45U
	DECFSZ	COUNT8,1	
	GOTO	READ_LOOP2
	MOVFW	TEMP_RD
	RETURN
READ_1_2	SETC
	GOTO	READ_L2_2	

RESET_PULSE	BCF	DQ2_PIN	; DQ = low
	BSF	STATUS,RP0	; !!Bank 1
	BCF	DQ2_PIN	; Set to output port
	BCF	STATUS,RP0	; !!Bank 0
	CALL	DELAY617U	; For reset pulse
	BSF	STATUS,RP0	; !!Bank 1
	BSF	DQ2_PIN	; DQ = Hi Z.
	BCF	STATUS,RP0 	; !!Bank 0 
	RETURN
PRES_PULSE    BSF	STATUS,RP0	; !!Bank 1
	BSF	DQ2_PIN	; DQ = Hi Z.
	BCF	STATUS,RP0	; !!Bank 0 
	CALL	DELAY15U
	BTFSC	DQ2_PIN	; Skip if clear
	GOTO	$-1	; Wait for presence pulse
	BTFSS	DQ2_PIN	; Skip if data line = float
	GOTO	$-1
	RETURN
WRITE	MOVWF	TEMP_WR
	MOVLW	.8
	MOVWF	COUNT8
WR_LOOP_2	RRF	TEMP_WR,1
	BC	SEND_1_2	; Branch if C = 1
SEND_0_2	BCF	DQ2_PIN
	BSF	STATUS,RP0	; !!Bank 1
	BCF	DQ2_PIN	; Set to output port
	BCF	STATUS,RP0	; !!Bank 0
	CALL	DELAY45U
L3_2	CALL	DELAY15U	
	BSF	STATUS,RP0	; !!Bank 1
	BSF	DQ2_PIN	; DQ = float
	BCF	STATUS,RP0	; !!Bank 0
	NOP
	DECF	COUNT8,1
	SKPZ
	GOTO	WR_LOOP_2
	RETURN
SEND_1_2	BCF	DQ2_PIN	
	BSF	STATUS,RP0	; !!Bank 1
	BCF	DQ2_PIN	; DQ = output
	BCF	STATUS,RP0	; !!Bank 0
	GOTO	L3_2
;-----------------------   END OF 1-WIRE  ------------------------------

;************************ 
; SEND TO RS-232  
;************************
TX	MOVWF	TX_BUFFER	; Send to RS-232 (PC or SLCD)
	MOVLW	.8	; 8N1,9600 BPS, not invert
	MOVWF	COUNT1
	CALL	DELAY_TX	; Delay for idle state
	BCF	TX_PIN	; Send start bit <logic 0>
	CALL	DELAY_TX	
LOOP_SEND	RRF	TX_BUFFER,1	; Send LSB first
	BTFSC	STATUS,C
	GOTO	L2
LOGIC_0	BCF	TX_PIN	; Send logic 0
LOGIC_1	CALL	DELAY_TX	
	DECFSZ	COUNT1,1
	GOTO	LOOP_SEND		
	BSF	TX_PIN	; Send stop bit<logic 1>
	CALL	DELAY_TX
	RETURN	
L2	BSF	TX_PIN	; Send logic 1	
	GOTO	LOGIC_1

;************************
; DELAY ROUTINES  
;************************
DELAY15U	MOVLW	.5	; Delay 15 uS
	MOVWF	D_VAR1
	DECFSZ	D_VAR1,1
	GOTO	$-1
	RETURN 
DELAY45U	MOVLW	.15           ; Delay 45 uS
	MOVWF	D_VAR1
	DECFSZ	D_VAR1,1
	GOTO	$-1
	NOP
	RETURN	
DELAY617U	MOVLW	.206	; Delay 617 uS
	MOVWF	D_VAR1
	DECFSZ	D_VAR1,1
	GOTO	$-1
	RETURN
DELAY_TX	MOVLW	.29	; Delay for RS-232
	MOVWF	D_VAR1	
	DECFSZ	D_VAR1,1
	GOTO	$-1
	RETURN		
DELAY	CLRF	D_VAR1	; Genaral purpose delay
	CLRF	D_VAR2
	MOVWF	D_VAR3	; D_VAR3 = 1-255
	DECFSZ	D_VAR1,1
	GOTO	$-1
	DECFSZ	D_VAR2,1
	GOTO	$-3
	DECFSZ	D_VAR3,1
	GOTO	$-5
	RETURN
;----------------------------  END OF DELAY  --------------------------------

BEEP	MOVLW	.125	; For beep loop
	MOVWF	BEEP_TEMP
BEEP_LOOP	BSF	PIEZO
	MOVLW	.25	; Delay for logic1
	CALL	DELAY_BEEP
	BCF	PIEZO
	MOVLW	.100 	; Delay for logic0
	CALL	DELAY_BEEP
	DECFSZ	BEEP_TEMP,1	
	GOTO	BEEP_LOOP
	RETURN
DELAY_BEEP	MOVWF	D_VAR1	
	DECFSZ	D_VAR1,1
	GOTO	$-1
	RETURN	
DELAY_SW	CLRF	D_VAR1	; Delay 10 mS
	MOVLW	.13
	MOVWF	D_VAR2
	DECFSZ	D_VAR1,1
	GOTO	$-1
	DECFSZ	D_VAR2,1
	GOTO	$-3
	RETURN
CLR_SLCD	MOVLW	0xFE	; Send SCLD command
	CALL	TX
	MOVLW	0x01	; Clear display command
	CALL	TX
	MOVLW	.1	; Delay ~200mS
	CALL	DELAY
	RETURN  
; *************************************************************************
; LOGIC AND ARITHMATIC ROUTINES
; *************************************************************************
; -----------------------    DIVISION 16*16   -----------------------------
; X input     = QUOTIENT_HIGH:QUOTIENT_LOW
; Y input     = R1HIGH:R1LOW
; output      = QUOTIENT_HIGH:QUOTIENT_LOW . REMAINDER_HIGH:REMAINDER_LOW
; Remainder   = X MOD Y		
; -------------------------------------------------------------------------		
DIV16	MOVLW	.16
	MOVWF	COUNTER
	CLRF	R1HIGH
	MOVLW	.10	; Fixed divider = 10
	MOVWF	R1LOW
	CLRF	REMAINDER_HIGH
	CLRF	REMAINDER_LOW
DIVIDE_LOOP	RLF	QUOTIENT_HIGH,0
	RLF	REMAINDER_LOW,1
	RLF	REMAINDER_HIGH,1
	MOVFW	R1LOW
	SUBWF	REMAINDER_LOW,1
	MOVFW	R1HIGH
	SKPC
	INCFSZ	R1HIGH,0
	SUBWF	REMAINDER_HIGH,1	
	SKPNC
	GOTO	RESULT
	MOVFW	R1LOW
	ADDWF	REMAINDER_LOW,1
	MOVFW	R1HIGH
	SKPNC
	INCFSZ	R1HIGH,0
	ADDWF	REMAINDER_HIGH,1	
	CLRC	
RESULT	RLF	QUOTIENT_LOW,1
	RLF	QUOTIENT_HIGH,1
	DECFSZ	COUNTER,1
	GOTO	DIVIDE_LOOP
	RETURN	
; ---------------------------   MULTIPLY 8*8   ----------------------------
; INPUT  = MULTIPLICAND * MULTIPLIER
; OUTPUT = PRODUCT_H:PRODUCT_L
; -------------------------------------------------------------------------
MUL8	MOVLW	.8
	MOVWF	COUNTER
	MOVLW	.18	; Fixed multiplier = D'18' 
	MOVWF	MULTIPLIER
	CLRF	PRODUCT_H
	CLRF	PRODUCT_L
	MOVFW	MULTIPLICAND
	CLRC
MUL8_LOOP	RRF	MULTIPLIER,1
	SKPNC
	ADDWF	PRODUCT_H,1
	RRF	PRODUCT_H,1
	RRF	PRODUCT_L,1
	DECFSZ	COUNTER,1
	GOTO	MUL8_LOOP
	RETURN
; -----------------------  CONVERT BINARY16 TO BCD  -----------------------
; INPUT  : H_BYTE:L_BYTE
; OUTPUT : R0:R1:R2
; -------------------------------------------------------------------------
BIN2BCD  	CLRC	  	 
	MOVLW   .16
	MOVWF   COUNT
	CLRF    R0
	CLRF    R1
	CLRF    R2
LOOP16  	RLF     L_BYTE,1
	RLF     H_BYTE,1
	RLF     R2, F
	RLF     R1, F
	RLF     R0, F
	DECFSZ  COUNT,1
	GOTO    ADJDEC
	RETLW   0
ADJDEC  	MOVLW   R2
	MOVWF   FSR
	CALL    ADJBCD
	MOVLW   R1
	MOVWF   FSR
	CALL    ADJBCD
	MOVLW   R0
	MOVWF   FSR
	CALL    ADJBCD
	GOTO    LOOP16
ADJBCD  	MOVLW   .3
	ADDWF   0,0
	MOVWF   TEMP
	BTFSC   TEMP,3          ; Test if result > 7
	MOVWF   0
	MOVLW   30
	ADDWF   0,0
	MOVWF   TEMP
	BTFSC   TEMP,7          ; Test if result > 7
	MOVWF   0               ; Save as MSD
	RETLW   0

	END
