;*************************************************************************************
; Filename :  PROJ11.ASM
; Description : Temperature alarm 
; MCU : 	PIC12F675
; Hardware :	AX-12 PIC12F675 Activity board
; Written by : Natthapol Wongsoonthornchai
; Copyright 2003 Innovative Experiment Co.,Ltd.
;*************************************************************************************

	TITLE	"PROJECT: Temperature alarm"
;****************************************************************
TH = +D'40'	; Setting here for TH alarm point  => [TL<TH<125] 
TL = +D'30' 	; Setting here for TL alarm point  => [-55<TL<TH]
;****************************************************************

;----------------------- Define ERROR messages ------------------------
   IF   TH >= D'125'		; TH out of range ?
      ERROR   "TH do not more than or equql 125 Celcius" ; If yes,then error
   	ENDIF
   	
   IF   TL <= -D'55'		; TL out of range ?
      ERROR   "TL do not less than or equql -55 Celcius" ; If yes,then error
	ENDIF   
   
   IF   TH <= TL		; Check TH and TL condition

	;========= TH and TL condition ===========
	;=	 TL<TH<125 	             =	
	;=	 -55<TL<TH	             =
	;=========================================
	
   ERROR 	"TH do not less than or equql TL!. Please set new value"
	ENDIF
;----------------------- End of checking ------------------------------

	PROCESSOR	12F675	; Assigned CPU.
	ERRORLEVEL	0,-302	; Issue all,suppress crossing bank0 (Address exceed 0x7F) 
	INCLUDE	P12F675.inc	; Use microchip's include file
	__IDLOCS	1820H 	; ID for this project
	__CONFIG	01C4H	; INTRC_osc,MCLR_off,WDT_off,BODEN_on,PWRT_on

     CBLOCK	0x20		; Assigned data RAM
	TEMP_WR,TEMP_RD,COUNT,COUNT1,COUNT8,D_VAR1,D_VAR2,D_VAR3
	H_BYTE,L_BYTE,R0,R1,R2,TX_BUFFER,BEEP_TEMP,TEMP,DP,ROUND
	TH_HIGH,TH_LOW,TL_HIGH,TL_LOW,TBL_POINTER,NUM_OF_CHAR,SIGN
	ALM_FLAG
	
	TEMP_MSB:2 ; Reserved 2 addr. for TEMP_MSB cal. & compare 
              TEMP_LSB:2 ; Reserved 2 addr. for TEMP_LSB cal. & compare 
     ENDC 
;************************************************
; Define port 
;************************************************
#DEFINE	DQ_PIN	GPIO,2	; DS1820 DQ pin
#DEFINE	TX_PIN	GPIO,1	; RS-232 pin
#DEFINE	SPK	GPIO,0	; Sound out


ORG	0x000
	GOTO	INIT_PORT	

ASCII_TABLE	ADDWF	PCL,1	; Convert BCD to ASCII
	DT	"0123456789"
SYMBOL	ADDWF	PCL,1	; Table for special symbol
CGRAM_ADDR00 	DT   	0x18,0x18,0x07,0x08,0x08,0x08,0x07,0x00	; Small C 
CGRAM_ADDR01	DT   	0x00,0x00,0x00,0x0C,0x12,0x12,0x1E,0x12	; Small A
CGRAM_ADDR02  DT	0x00,0x00,0x00,0x12,0x12,0x1E,0x12,0x12	; Small H
CGRAM_ADDR03	DT	0x00,0x00,0x00,0x10,0x10,0x10,0x10,0x1E 	; Small L
	
;****************************
; Initial port 
;****************************
INIT_PORT	MOVLW	.7	; Turn-off comparator
	MOVWF	CMCON		
	BSF	TX_PIN	; Set latch for TX idle state
	BANKSEL	ANSEL	; Bank 1			
	BCF	TX_PIN	; Set as output <For RS-232>
	BCF	SPK	; Set as output <For Speaker>
	CALL	0x3FF	; Load OSC. value
	MOVWF	OSCCAL	; Calibratibg here
	CLRF	ANSEL	; GP4,GP2:GP0 = digital port
	BANKSEL	GPIO	; Bank 0
	CALL	READ_TEMP	; Removed DS1820'S first result<D'85'>
	MOVLW	.5
	CALL	DELAY	; Delay 1 sec for SLCD start-up
	CALL	CLR_SLCD
	
;**************************
; Write special symbol 
;**************************	
WR_CHAR	MOVLW	0xFE	; Send SLCD command prefix
	CALL	TX	
	MOVLW	0x40	; Write CGRAM command
	CALL	TX
	MOVLW	.00
	MOVWF	TBL_POINTER	; Table address = 0
	MOVLW	.8*.4
	MOVWF	NUM_OF_CHAR	; Write 32 lines for 4 symbols
WR_CGRAM	MOVFW	TBL_POINTER
	CALL	SYMBOL	; Look-up table
	CALL	TX
	INCF	TBL_POINTER,1
	DECFSZ	NUM_OF_CHAR,1	; Skip if completed
	GOTO	WR_CGRAM
	
	

;****************************
; Setting MSB of TH 
;****************************
    IF TH < 0 
	MOVLW	0xFF
	MOVWF	TH_HIGH 	; MSB = 0xFF if negative temperature
    ELSE
	CLRF	TH_HIGH	; MSB = 0x00 if positive temperature
        	ENDIF
;****************************
; Setting MSB of TL 
;****************************

    IF TL < 0 
	MOVLW	0xFF
	MOVWF	TL_HIGH 	; MSB = 0xFF if negative temperature
    ELSE
	CLRF	TL_HIGH	; MSB = 0x00 if positive temperature
        	ENDIF
;****************************************************************************
	
	MOVFW	TH_HIGH	; Calculate TH point for show
	MOVWF	TEMP_MSB	; Load MSB
	MOVLW	TH*2	
	MOVWF	TH_LOW	
	MOVWF	TEMP_LSB	; Load LSB
	CALL	TEMP_CAL	; Cal. temperature in C
	MOVLW	0xFE	; Set SLCD line 2/1
	CALL	TX
	MOVLW	0xC0	
	CALL	TX	
	MOVLW	"T"	; Show TH point on SLCD
	CALL	TX
	MOVLW	0x02	; Load small H
	CALL	TX	; Show small H
	CALL	SHOW	; Show TH
	MOVLW	","
	CALL	TX

	MOVFW	TL_HIGH	; Calculate TL point for show
	MOVWF	TEMP_MSB	; Load MSB
	MOVLW	TL*2	
	MOVWF	TL_LOW	; Load LSB
	MOVWF	TEMP_LSB
	CALL	TEMP_CAL	; Cal. temperature in C
	MOVLW	"T"	; Show TL point on SLCD
	CALL	TX
	MOVLW	0x03	; Load small L
	CALL	TX	; Show small L
	CALL	SHOW	; Show TL

	
;****************************** Main program ****************************

MAIN_LOOP	CALL	READ_TEMP	; Read ambient temperature (TA)
	CALL	TEMP_CAL	; Calculate temperature in C
	MOVLW	0xFE	; Set SLCD line 1/1
	CALL	TX
	MOVLW	0x80	
	CALL	TX	
	MOVLW	"T"	; Show TH point on SLCD
	CALL	TX
	MOVLW	0x01	; Load small A
	CALL	TX	; Show small A
	CALL	SHOW	; Show TH
	MOVLW	"."
	CALL	TX
	MOVFW	DP	; Load decimal point (0 or 5)
	CALL	TX	; Show decimal point
	MOVLW	0x00	; Load small C
	CALL	TX	; Show small C

	CALL	CMP_TH_POINT	; Check TH alarm point
	BTFSC	ALM_FLAG,0	; Skip if no alarm
	CALL	TH_BEEP	; Sound for TH alarm 
	CALL	CMP_TL_POINT	; Check TL alarm point
	BTFSC	ALM_FLAG,0	; Skip if no alarm
	CALL	TL_BEEP	; Sound for TL alarm 
	MOVLW	.3	; Delay ~0.6 sec
	CALL	DELAY
	GOTO	MAIN_LOOP

;*****************************************
; Alarm point compare among TA,TH and TL 
;*****************************************
CMP_TH_POINT  TSTF	TEMP_MSB+1	; Check TA range(look MSB), pos or neg temp.? 
	BNZ	NEG_RANGE1	; Jump to negative range
POS_RANGE1	TSTF	TH_HIGH	; Check TH range(look MSB), pos or neg temp.? 
	BNZ	NEG_TH1	; Jump to TH negative range	
POS_TH1	MOVFW	TEMP_LSB+1	; Restored LSB of TA
	SUBWF	TH_LOW,0	; Compare(TH-TA), TH < TA ? 
	BSF	ALM_FLAG,0	; Yes,set alarm flag
	SKPC		; Skip if TH=>TA
	RETURN		; Alarm point occured here (TH<TA)	
	BCF	ALM_FLAG,0	; Clear alarm flag 
	RETURN		; No alarm point occur then return
NEG_TH1	BSF	ALM_FLAG,0	; Alarm point occured (TA>TH)
	RETURN		
NEG_RANGE1	TSTF	TH_HIGH	; Check TH range(look MSB), pos or neg temp.? 
	BNZ	NEG_TH2	; Jump to TH negative range	
POS_TH2	BCF	ALM_FLAG,0	; No alarm point occur (TA<TH),then return
	RETURN
NEG_TH2	MOVFW	TEMP_LSB+1	; Restored LSB of TA
	SUBWF	TH_LOW,0	; Compare(TH-TA), TH < TA ? 
	BSF	ALM_FLAG,0	; Yes,set alarm flag
	SKPC		; Skip if TH=>TA
	RETURN		; Alarm point occured here (TH<TA)	
	BCF	ALM_FLAG,0	; Clear alarm flag 
	RETURN		; No alarm point occur then return

CMP_TL_POINT	TSTF	TEMP_MSB+1	; Check TA range(look MSB), pos or neg temp.? 
	BNZ	NEG_RANGE2	; Jump to negative range
POS_RANGE3	TSTF	TL_HIGH	; Check TL range(look MSB), pos or neg temp.? 
	BNZ	NEG_TL1	; Jump to TL negative range	
POS_TL1	MOVFW	TL_LOW	; Restored LSB of TL
	SUBWF	TEMP_LSB+1,0	; Compare(TA-TL), TA < TL ? 
	BSF	ALM_FLAG,0	; Yes,set alarm flag
	SKPC		; Skip if TL=>TA
	RETURN		; Alarm point occured here (TA<TL)	
NEG_TL1	BCF	ALM_FLAG,0	; Clear alarm flag 
	RETURN		; No alarm point occur then return
NEG_RANGE2	TSTF	TL_HIGH	; Check TL range(look MSB), pos or neg temp.? 
	BNZ	NEG_TL2	; Jump to TL negative range	
POS_TL2	BSF	ALM_FLAG,0	; Alarm point occured here (TA<TL)	
	RETURN
NEG_TL2	MOVFW	TL_LOW	; Restored LSB of TL
	SUBWF	TEMP_LSB+1,0	; Compare(TA-TL), TA < TL ? 
	BSF	ALM_FLAG,0	; Yes,set alarm flag
	SKPC		; Skip if TA=>TL
	RETURN		; Alarm point occured here (TA<TL)	
	BCF	ALM_FLAG,0	; Clear alarm flag 
	RETURN		; No alarm point occur then return
;************************** End of main program ******************************

TEMP_CAL	MOVLW	" " 	; Show white space for positive temp
	MOVWF	SIGN		
	TSTF	TEMP_MSB	; Test file, pos or neg temp?
	BZ	POS_TEMP	; Jump if positive temp
NEG_TEMP	MOVLW	"-"
	MOVWF	SIGN	; Show minus sign for negative temp.
	NEGF	TEMP_LSB,1	; Change to 2's complement
POS_TEMP	CLRC		; Clear carry flag before rotate	
	RRF	TEMP_LSB,1	; Temperature = TEMP_LSB / 2
	MOVLW	"0"
	MOVWF	DP	; For show 0.0 degree
	MOVLW	"5"	; For show 0.5 degree
	SKPNC		; Skip if carry=0 
	MOVWF	DP	
	CLRF	H_BYTE	; Load BCD input
	MOVFW	TEMP_LSB
	MOVWF	L_BYTE
	CALL	BIN2BCD	; Adjust result to BCD format
	CALL	FIND_NUMBER	; Find ASCII for SLCD 
	RETURN

FIND_NUMBER	MOVFW	R1	
	CALL	ASCII_TABLE	; Find ASCII number
	MOVWF	R0	; R0 = hundred unit
	ADDLW	-0x30	; w = w-0x30 ('0')
	BNZ	CONVERT1	; Jump if w <> 0
	MOVLW	' '	; Show white space if temp <100 C
	MOVWF	R0	
	SWAPF	R2,0
	ANDLW	B'00001111'	; Mask bit
	CALL	ASCII_TABLE
	MOVWF	R1	; R1 = ten digit
	ADDLW	-0x30	; w = w-0x30 ('0')
	BNZ	CONVERT2	; Jump if w <> 0
	MOVLW	' '	; Show white spaceHOW if temp < 10 C
	MOVWF	R1	
	GOTO	CONVERT2
CONVERT1	SWAPF	R2,0	
	ANDLW	B'00001111'	; Mask bit
	CALL	ASCII_TABLE
	MOVWF	R1	; R1 = ten digit
CONVERT2	MOVFW	R2
	ANDLW	B'00001111'	; Mask bit
	CALL	ASCII_TABLE
	MOVWF	R2	; R2 = unit degree
	RETURN

SHOW	MOVLW	":"
	CALL	TX
	MOVFW	SIGN
	CALL	TX
	MOVFW	R0	; Show hundred
	CALL	TX
	MOVFW	R1	; Show ten
	CALL	TX
	MOVFW	R2	; Show unit
	CALL	TX
	RETURN
;**************************
; 1-WIRE ROUTINE 
;**************************
READ_TEMP	CALL	RESET_PULSE
	CALL	PRES_PULSE
	MOVLW	0xCC	; Send skip rom command
	CALL	WRITE
	MOVLW	0x44	; Send convert temp command
	CALL	WRITE
	CALL	RESET_PULSE
	CALL	PRES_PULSE
	MOVLW	0xCC	; Send skip rom command
	CALL	WRITE
	MOVLW	0xBE	; Send read temp command
	CALL	WRITE	
	CALL	READ	; Read LSB (TL)
	MOVWF	TEMP_LSB	; Stored LSB for calculate
	MOVWF	TEMP_LSB+1	; Copy LSB for compare	
	CALL	READ	; Read MSB (TH)
	MOVWF	TEMP_MSB	; Stored MSB for calculate
	MOVWF	TEMP_MSB+1	; Copy MSB for compare
	RETURN
READ	MOVLW	.8	; Number of loop = 8
	MOVWF	COUNT8
READ_LOOP2	BCF	DQ_PIN
	BSF	STATUS,RP0	; Bank 1
	BCF	DQ_PIN	; DQ = OUTPUT
	BCF	STATUS,RP0	; Bank 0
	GOTO	$+1
	BSF	STATUS,RP0	; Bank 1
	BSF	DQ_PIN	; Set to input port
	BCF	STATUS,RP0	; Bank 0
	BTFSC	DQ_PIN	; Read data at this pin
	GOTO	READ_1_2
READ_0_2	CLRC		; Clear carry flag
READ_L2_2	RRF	TEMP_RD,1	; Rotate right for receive LSB first 
	CALL	DELAY45U
	DECFSZ	COUNT8,1	
	GOTO	READ_LOOP2
	MOVFW	TEMP_RD	; Load result into w
	RETURN
READ_1_2	SETC		; Set carry flag
	GOTO	READ_L2_2	

RESET_PULSE	BCF	DQ_PIN	; DQ = low
	BSF	STATUS,RP0	; Bank 1
	BCF	DQ_PIN	; Set to output port
	BCF	STATUS,RP0	; Bank 0
	CALL	DELAY617U	; For reset pulse
	BSF	STATUS,RP0	; Bank 1
	BSF	DQ_PIN	; DQ = Hi Z.
	BCF	STATUS,RP0 	; Bank 0 
	RETURN
PRES_PULSE    BSF	STATUS,RP0	; Bank 1
	BSF	DQ_PIN	; DQ = Hi Z.
	BCF	STATUS,RP0	; Bank 0 
	CALL	DELAY15U
	BTFSC	DQ_PIN	; Skip if clear
	GOTO	$-1	; Wait for presence pulse
	BTFSS	DQ_PIN	; Skip if data line = float
	GOTO	$-1
	RETURN
WRITE	MOVWF	TEMP_WR	; Load data from w 
	MOVLW	.8	; Number of loop for eight bit of data
	MOVWF	COUNT8
WR_LOOP_2	RRF	TEMP_WR,1	; Rotare right for send LSB first
	BC	SEND_1_2	; Branch if C = 1
SEND_0_2	BCF	DQ_PIN
	BSF	STATUS,RP0	; Bank 1
	BCF	DQ_PIN	; Set to output port
	BCF	STATUS,RP0	; Bank 0
	CALL	DELAY45U
L3_2	CALL	DELAY15U	
	BSF	STATUS,RP0	; Bank 1
	BSF	DQ_PIN	; DQ = float
	BCF	STATUS,RP0	; Bank 0
	NOP
	DECFSZ	COUNT8,1	; Skip if write completed
	GOTO	WR_LOOP_2
	RETURN
SEND_1_2	BCF	DQ_PIN	
	BSF	STATUS,RP0	; Bank 1
	BCF	DQ_PIN	; DQ = output
	BCF	STATUS,RP0	; Bank 0
	GOTO	L3_2
;************************* END OF 1-WIRE ******************************

;************************* 
; SEND TO RS-232 
;*************************
TX	MOVWF	TX_BUFFER	; Send to RS-232 (PC or SLCD)
	MOVLW	.8	; 8N1,9600 BPS, not invert
	MOVWF	COUNT1
	CALL	DELAY_TX	; Delay for idle state
	BCF	TX_PIN	; Send start bit <logic 0>
	CALL	DELAY_TX	
LOOP_SEND	RRF	TX_BUFFER,1	; Send LSB first
	BTFSC	STATUS,C
	GOTO	L2
LOGIC_0	BCF	TX_PIN	; Send logic 0
LOGIC_1	CALL	DELAY_TX	
	DECFSZ	COUNT1,1
	GOTO	LOOP_SEND		
	BSF	TX_PIN	; Send stop bit<logic 1>
	CALL	DELAY_TX
	RETURN	
L2	BSF	TX_PIN	; Send logic 1	
	GOTO	LOGIC_1

;************************
; DELAY ROUTINES  
;************************

DELAY15U	MOVLW	.5	; Delay 15 uS
	MOVWF	D_VAR1
	DECFSZ	D_VAR1,1
	GOTO	$-1
	RETURN 
DELAY45U	MOVLW	.15           ; Delay 45 uS
	MOVWF	D_VAR1
	DECFSZ	D_VAR1,1
	GOTO	$-1
	NOP
	RETURN	
DELAY617U	MOVLW	.206	; Delay 617 uS
	MOVWF	D_VAR1
	DECFSZ	D_VAR1,1
	GOTO	$-1
	RETURN
DELAY_TX	MOVLW	.29	; Delay for RS-232
	MOVWF	D_VAR1	
	DECFSZ	D_VAR1,1
	GOTO	$-1
	RETURN		
DELAY	CLRF	D_VAR1	; Genaral purpose delay
	CLRF	D_VAR2
	MOVWF	D_VAR3	; D_VAR3 = 1-255
	DECFSZ	D_VAR1,1
	GOTO	$-1
	DECFSZ	D_VAR2,1
	GOTO	$-3
	DECFSZ	D_VAR3,1
	GOTO	$-5
	RETURN
;*************************** END OF DELAY *****************************

TH_BEEP	MOVLW	.125	; For beep loop
	MOVWF	BEEP_TEMP
TH_BEEP_LOOP	BSF	SPK
	MOVLW	.25	; Delay for logic1
	CALL	DELAY_BEEP
	BCF	SPK
	MOVLW	.100 	; Delay for logic0
	CALL	DELAY_BEEP
	DECFSZ	BEEP_TEMP,1	
	GOTO	TH_BEEP_LOOP
	RETURN
TL_BEEP	MOVLW	.125	; For beep loop
	MOVWF	BEEP_TEMP
TL_BEEP_LOOP	BSF	SPK
	MOVLW	.00	; Delay for logic1
	CALL	DELAY_BEEP
	BCF	SPK
	MOVLW	.00	; Delay for logic0
	CALL	DELAY_BEEP
	DECFSZ	BEEP_TEMP,1	
	GOTO	TL_BEEP_LOOP
	RETURN

DELAY_BEEP	MOVWF	D_VAR1	
	DECFSZ	D_VAR1,1
	GOTO	$-1
	RETURN	
DELAY_SW	CLRF	D_VAR1	; Delay 10 mS
	MOVLW	.13
	MOVWF	D_VAR2
	DECFSZ	D_VAR1,1
	GOTO	$-1
	DECFSZ	D_VAR2,1
	GOTO	$-3
	RETURN
CLR_SLCD	MOVLW	0xFE	; Send 0xFE before send SCLD's command
	CALL	TX
	MOVLW	0x01	; Clear display command
	CALL	TX
	MOVLW	.1	; Delay ~200mS
	CALL	DELAY
	RETURN  
;*********************************** 
; Convert binary16 to BCD5 
; Input(binary16)  : H_BYTE:L_BYTE
; Output(BCD5)     : R0:R1:R2
;***********************************
BIN2BCD  	CLRC	  	 
	MOVLW   .16
	MOVWF   COUNT
	CLRF    R0
	CLRF    R1
	CLRF    R2
LOOP16  	RLF     L_BYTE,1
	RLF     H_BYTE,1
	RLF     R2, F
	RLF     R1, F
	RLF     R0, F
	DECFSZ  COUNT,1
	GOTO    ADJDEC
	RETURN
ADJDEC  	MOVLW   R2
	MOVWF   FSR
	CALL    ADJBCD
	MOVLW   R1
	MOVWF   FSR
	CALL    ADJBCD
	MOVLW   R0
	MOVWF   FSR
	CALL    ADJBCD
	GOTO    LOOP16
ADJBCD  	MOVLW   .3
	ADDWF   0,0
	MOVWF   TEMP
	BTFSC   TEMP,3          ; Test if result > 7
	MOVWF   0
	MOVLW   30
	ADDWF   0,0
	MOVWF   TEMP
	BTFSC   TEMP,7          ; Test if result > 7
	MOVWF   0               ; Save as MSD
	RETURN

	END	
