;*************************************************************************************
; Filename :  PROJ12.ASM
; Description : Read ID from DS1990A serial number iButton
; MCU : 	PIC12F675
; Hardware :	AX-12 PIC12F675 Activity board
; Written by : Natthapol Wongsoonthornchai
; Copyright 2003 Innovative Experiment Co.,Ltd.
;*************************************************************************************
	#DEFINE	USE_RS232	; Use this line if would like to show on PC screen
		
	LIST	P=12F675
	INCLUDE	P12F675.INC	; Use Microchip include file
	__CONFIG	0x01A4	; INTRC OSC, WDT off, MCLR on
	__IDLOCS 	0x1990	; Specific ID = 1990H
	ERRORLEVEL 	-302 	; Suppressed warning M[302]
     CBLOCK	0x20		; GP RAM		
         FAMILY,ID0,ID1,ID2,ID3,ID4,ID5,CRC,D_VAR1,D_VAR2,D_VAR3
            BUFFER,TX_BUFFER,RX_BUFFER,COUNT1,TBL_POINTER
	   TEMP_WR,TEMP_RD,COUNT8,D_TEMP,NUM_OF_CHAR
	     ENDC

LF	EQU	.10 	; line feed 
FF	EQU	.12	; Form feed
CR	EQU	.13 	; Carriage return

#DEFINE	DQ_PIN	GPIO,2	; GP2 = data pin
#DEFINE	TX_PIN	GPIO,1	; GP1 = TX for RS-232

	ORG	.0000	; Start here

INIT_PORT	MOVLW	.7	
	MOVWF	CMCON	; Turn-off comparator
	BSF	TX_PIN	; Set latch for RS-232 idle state
	BANKSEL	TRISIO	; Bank 1
	BCF	TX_PIN	; GP1 = output
	CLRF	ANSEL	; GP4-GP0 = digital i/o
	CALL	.1023	; Call OSC value
	MOVWF	OSCCAL	; Calibrate OSC
	BANKSEL	GPIO	; Bank 0

    IFNDEF	USE_RS232		; Show on SLCD screen	
	MOVLW	.6	; Delay 1 second for SLCD start-up
	CALL	DELAY	
	MOVLW	0xFE	; SLCD command prefix
	CALL	TX
	MOVLW	0x01	; Clear SLCD
	CALL	TX
    ENDIF
    
READ_ID	CALL	RESET_PULSE	; Send reset pulse
	CALL	PRES_PULSE	; Receive presence pulse (ACK.) 
	MOVLW	0x33	; Send read ROM command for DS1990A 
	CALL	OW_WRITE	
	CALL	DELAY15U	; Delay 15 microsecond

	CALL	OW_READ	; Read family code
	MOVFW	TEMP_RD	
	MOVWF	FAMILY	; Save to family variableiable

	CALL	OW_READ	; Read serial number, ID0
	MOVFW	TEMP_RD	
	MOVWF	ID0	; Save to ID0 variable

	CALL	OW_READ	; Read ID1
	MOVFW	TEMP_RD	
	MOVWF	ID1	; Save to ID1 variable

	CALL	OW_READ	; Read ID2
	MOVFW	TEMP_RD
	MOVWF	ID2	; Save to ID2 variable

	CALL	OW_READ	; Read ID3
	MOVFW	TEMP_RD
	MOVWF	ID3	; Save to ID3 variable

	CALL	OW_READ	; Read ID4
	MOVFW	TEMP_RD
	MOVWF	ID4	; Save to ID4 variable

	CALL	OW_READ	; Read ID5
	MOVFW	TEMP_RD
	MOVWF	ID5	; Save to ID5 variable

	CALL	OW_READ	; Read CRC
	MOVFW	TEMP_RD
	MOVWF	CRC	; Save to CTC variable

     IFNDEF	USE_RS232		; If show on SLCD screen
  	MOVLW	0xFE	; LCD command prefix
  	CALL	TX
  	MOVLW	0x80	; Set line1/1
  	CALL	TX
     ENDIF	

     IFNDEF	USE_RS232		; If show on PC screen	
	MOVLW	LF	; Send line feed
	CALL	TX
	MOVLW	CR	; Send carriage return
	CALL	TX
     ENDIF		

 	MOVLW	.0	; Show text string 
	MOVWF	TBL_POINTER	; Table address = 0
	MOVLW	.12
	MOVWF	NUM_OF_CHAR	; Show "Device's ID="
TEXT_LOOP	MOVFW	TBL_POINTER
	CALL	TEXT_TABLE
	CALL	TX
	INCF	TBL_POINTER,1
	DECFSZ	NUM_OF_CHAR,1
	GOTO	TEXT_LOOP 	
	
     IFNDEF	USE_RS232		; If show on SLCD screen
  	MOVLW	0xFE	; LCD command prefix
  	CALL	TX
  	MOVLW	0xC0	; Set line2/1
  	CALL	TX
     ENDIF	
     
 	CALL	SHOW	; 
 	MOVLW	.6	; Delay 1 second
 	CALL	DELAY
	GOTO	READ_ID

TABLE	ADDWF	PCL,1	; Number 0-F
	DT            "0123456789ABCDEF"	

TEXT_TABLE	ADDWF	PCL,1	; Text table
	DT	"Device's ID="
SHOW
	MOVFW	CRC
	CALL	DEBUG
	MOVFW	ID5
	CALL	DEBUG
	MOVFW	ID4
	CALL	DEBUG	
	MOVFW	ID3
	CALL	DEBUG
	MOVFW	ID2
	CALL	DEBUG	
	
	MOVFW	ID1
	CALL	DEBUG
	MOVFW	ID0
	CALL	DEBUG
	MOVFW	FAMILY
	CALL	DEBUG
     IFNDEF	USE_RS232		; If show on PC screen
	MOVLW	LF	; Send line feed
	CALL	TX
	MOVLW	CR	; Send carriage return
	CALL	TX 
     ENDIF	
	RETURN
 	
RESET_PULSE	BCF	DQ_PIN  	; Logic 0
	BSF	STATUS,RP0	; Bank 1
	BCF	DQ_PIN 	; Port = output
	BCF	STATUS,RP0	; Bank 0
	CALL	DELAY617U	; For reset pulse
	BSF	STATUS,RP0	; Bank 1
	BSF	DQ_PIN 	; Port = high impedance
	BCF	STATUS,RP0 	; Bank 0 
	RETURN

PRES_PULSE    BSF	STATUS,RP0	; Bank 1
	BSF	DQ_PIN 	; Port = high impedance
	BCF	STATUS,RP0	; Bank 0 
	CALL	DELAY15U
	BTFSC	DQ_PIN 	; Skip if clear
	GOTO	$-1	; Wait for presence pulse
	BTFSS	DQ_PIN 	; Skip if DQ = idle
	GOTO	$-1
	RETURN
	
OW_WRITE	MOVWF	TEMP_WR
	MOVLW	.8
	MOVWF	COUNT8
WR_LOOP	RRF	TEMP_WR,1
	BC	WR_1	; Branch if C = 1
WR_0	BCF	DQ_PIN 
	BSF	STATUS,RP0	; Bank 1
	BCF	DQ_PIN 	; Port = output
	BCF	STATUS,RP0	; Bank 0
	CALL	DELAY15U
	CALL	DELAY45U	
	BSF	STATUS,RP0	; Bank 1
	BSF	DQ_PIN 	; Port = float
	BCF	STATUS,RP0	; Bank 0
	NOP
	GOTO	$+1
	DECFSZ	COUNT8,1
	GOTO	WR_LOOP
	RETURN
	
WR_1	BCF	DQ_PIN 	
	BSF	STATUS,RP0	; Bank 1
	BCF	DQ_PIN 	; Port = output
	BCF	STATUS,RP0	; Bank 0
	CALL	DELAY15U	
	BSF	STATUS,RP0	; Bank 1
	BSF	DQ_PIN 	; Port = float
	BCF	STATUS,RP0	; Bank 0
	CALL	DELAY45U
	GOTO	$+1
	NOP
	DECFSZ	COUNT8,1
	GOTO	WR_LOOP
	RETURN		

OW_READ	MOVLW	.8
	MOVWF	COUNT8
READ_LOOP	BCF	DQ_PIN 
	BSF	STATUS,RP0	; Bank 1
	BCF	DQ_PIN 	; Port = output
	BCF	STATUS,RP0	; Bank 0
	GOTO	$+1
	GOTO	$+1	
	BSF	STATUS,RP0	; Bank 1
	BSF	DQ_PIN 	; Port = input
	BCF	STATUS,RP0	; Bank 0
READ_LOOP2	BTFSC	DQ_PIN 	; Read data at DQ pin
	GOTO	READ_1
READ_0	CLRC		; Carry = 0
READ_1_2	RRF	TEMP_RD,1	
	CALL	DELAY45U
	DECFSZ	COUNT8,1
	GOTO	READ_LOOP 
	RETURN	
READ_1	SETC		; Carry = 1
	GOTO	READ_1_2

DEBUG	MOVWF	D_TEMP	; Find number
	SWAPF	D_TEMP,0
	ANDLW	B'00001111'	; Mask bit
	CALL	TABLE
	CALL	TX
	MOVFW	D_TEMP
	ANDLW	B'00001111'	; Mask bit
	CALL	TABLE
	CALL	TX	
	RETURN

TX	MOVWF	TX_BUFFER	; 8N1,9600 bps,not invert
	MOVLW	.8
	MOVWF	COUNT1
	BCF	TX_PIN	; Start bit
	NOP		; 1 microsecond
	GOTO	$+1	; 2 microsecond
	CALL	DELAY_TX	; 2 microsecond
LOOP_SEND	RRF	TX_BUFFER,1	; Send LSB first
	SKPNC
	GOTO	SEND_1
SEND_0	NOP
	BCF	TX_PIN	; LOGIC 0
SEND_1_2	CALL	DELAY_TX	; 2 microsecond
	DECFSZ	COUNT1,1
	GOTO	LOOP_SEND		
	BSF	TX_PIN	; Stop bit <HIGH>
	NOP
	GOTO	$+1
	CALL	DELAY_TX
	RETURN	

SEND_1	BSF	TX_PIN	; Logic 1
	GOTO	SEND_1_2

;************************
; DELAY SUBROUTINES 
;************************
DELAY_TX	MOVLW	.29	; Delay for RS-232
	MOVWF	D_VAR1	
	DECFSZ	D_VAR1,1
	GOTO	$-1
	RETURN		
DELAY15U	MOVLW	.5	; Delay 15 microsecond
	MOVWF	D_VAR1
	DECFSZ	D_VAR1,1
	GOTO	$-1
	RETURN
DELAY45U	MOVLW	.15           ; Delay 45 microsecond
	MOVWF	D_VAR1
	DECFSZ	D_VAR1,1
	GOTO	$-1
	NOP
	RETURN	
DELAY617U	MOVLW	.206	; Delay 617 microsecond
	MOVWF	D_VAR1
	DECFSZ	D_VAR1,1
	GOTO	$-1
	RETURN
DELAY	MOVWF	D_VAR3	; General purpose delay
	CLRF	D_VAR2
	CLRF	D_VAR1
	DECFSZ	D_VAR1,1
	GOTO	$-1
	DECFSZ	D_VAR2,1
	GOTO	$-3
	DECFSZ	D_VAR3,1
	GOTO	$-5
	RETURN
	
	END
