;*************************************************************************************
; Filename :  PROJ13.ASM
; Description : Electronic key with iButton DS1990A
; MCU : 	PIC12F675
; Hardware :	AX-12 PIC12F675 Activity board
; Written by : Natthapol Wongsoonthornchai
; Copyright 2003 innovative Experiment Co.,Ltd.
;*************************************************************************************

	LIST	P=12F675
	ERRORLEVEL	-302	
	INCLUDE	P12F675.INC	; Use Microchip include fiel
	__CONFIG	0x11C4	; INTRC OSC, WDT off, MCLR off 
	__IDLOCS	1990	; MCU ID = 1990 
      CBLOCK	0x20 + .16		; Reserved first 16 bytes for READ & CRC cal.
      D_VAR1,D_VAR2,D_VAR3,COUNT1,COUNT2,COUNT8,TBL_POINTER
      BUFFER,TX_BUFFER,D_TEMP,TEMP_WR,TEMP_RD,CRC,TEMP,NUM_OF_CHAR	
      COUNT3,ID_TEMP1,SOUND_TEMP
      ENDC
CRC_RD	EQU	0x2E	; CRC temp (READ VALUE)

; RAM address 0x20,0x21 = family code   (LSB)  ; TX/RX => LSB first
; RAM address 0x22,0x23 = serial number byte0
; RAM address 0x24,0x25 = serial number byte1
; RAM address 0x26,0x27 = serial number byte2
; RAM address 0x28,0x29 = serial number byte3
; RAM address 0x2A,0x2B = serial number byte4 
; RAM address 0x2C,0x2D = serial number byte5
; RAM address 0x2E,0x2F = CRC value     (MSB)
; from above even address = ID that use for compare with internal EEPROM
; and odd address = ID's temp that use for CRC calculation

#DEFINE	DQ_PIN	GPIO,2	; GP2 = data pin
#DEFINE	TX_PIN	GPIO,1	; GP1 = TX for RS-232
#DEFINE	SPKR	GPIO,0	; GP0 = sound out
#DEFINE	SW3	GPIO,5	; Use as ENTER switch
#DEFINE	SW2	GPIO,3	; Use as CALCEL switch

	ORG	0x2108	; Point to EEPROM address 0x08
	DE	0x00	; Indicator address <00 = no password>
	ORG	.0000	; Prpgram start here
	GOTO	INIT_PORT	
;**************************** 
; LOOK-UP TABLE 
;****************************
TABLE1	ADDWF	PCL,1	; Number 0-F
	DT  "0123456789ABCDEF"	
TABLE2	ADDWF	PCL,1
	DT  "No Password !!Please set your password" ; Table address = .00-.37
	DT  "Are you sure toaccept this ID ?" 	   ; Table address = .38-.68
TABLE3	ADDWF	PCL,1	
	DT  "Are you accept this ID? "
	DT  "If yes press SW3, if no press SW2."
	DT  "                "	; Fill space
TABLE4	ADDWF	PCL,1	
	DT  "Yes or No ?"	
TABLE5	ADDWF	PCL,1	
	DT  "Cancelled ID!"	
TABLE6	ADDWF	PCL,1	
	DT  "Accepted ID     "	
TABLE7	ADDWF	PCL,1
	DT  "ELECTRONICS KEYWhat's your ID?"
	DT  "ID PASSEDID FAILED!"
;*******************************************************************************
INIT_PORT	MOVLW	.7	; Turn-off comparator
	MOVWF	CMCON	
	BSF	TX_PIN	; Set latch for RS-232 idel state 
	BANKSEL	TRISIO	; Bank 1
	BCF	TX_PIN	; GP1 = output <RS-232 TX pin>
	BCF	SPKR	; GP0 = output <for speaker>
	CLRF	ANSEL	; GP4-GP0 = digital i/o
	CALL	.1023	; Call OSC value
	MOVWF	OSCCAL	; Calibrate OSC
	BANKSEL	GPIO	; Bank 0
	MOVLW	.6	; delay 1 second for SLCD set-up
	CALL	DELAY			

CHK_PASSWORD	MOVLW	0x08	; Point to indicator address
	BANKSEL	EEADR	; Bank 1
	MOVWF	EEADR
	CALL	READ_EE	; Read indicator data
	XORLW	0x00	; Have password stored in EEPROM ?
	BNZ	MAIN	; Yes, then jump to main
	CALL	NO_PASSWORD	; No, show "No Password!!" on SLCD
	MOVLW	.6
	CALL	DELAY	; delay 1 second
	CALL	READ_ID	; Read slave ID
	CALL	CLR_DISPLAY	; Clear SLCD
	CALL	SHOW	; Show slave ID on SLCD
	CALL	SOUND	; Send beeep
	MOVLW	.10	; Delay 2 seconds
	CALL	DELAY
	CALL	CONFIRM_ID1	; Ask, for confirm this ID 
WAIT_SW	BTFSS	SW3	; Skip if not press
	GOTO	ENTER	; if accepted password
	BTFSC	SW2	; Skip if pressed <CANCELLED> 
	GOTO	WAIT_SW	; Wait for press SW3 or SW2	
	CALL	SET_LINE2	; LCD line2/1
	MOVLW	.0
	MOVWF	TBL_POINTER	; Table address = 0
	MOVLW	.13
	MOVWF	NUM_OF_CHAR	; Show "Cancelled ID!" on LCD LINE 2/1
MESSAGE5	MOVFW	TBL_POINTER
	CALL	TABLE5	; Look-up table
	CALL	TX
	INCF	TBL_POINTER,1
	DECFSZ	NUM_OF_CHAR,1
	GOTO	MESSAGE5
	CALL	SOUND	; Send beep
	MOVLW	.10	; Delay 2 seconds
	CALL	DELAY
	CALL	CLR_DISPLAY	; Clear SLCD
	GOTO	CHK_PASSWORD	; Start again

ENTER	CALL	SOUND	; If pressed S3
	MOVLW	0x22
	MOVWF	FSR	; Point to ID (LSB)
	BANKSEL	EEADR	; Bank 1
	MOVLW	0x00	; Point to EEPROM address
	MOVWF	EEADR	
WRITE_LOOP	MOVFW	INDF	; Load ID to W
	MOVWF	EEDATA	
	CALL	WRITE_EE	; Write ID to EEPROM (48 bits)
	INCF	FSR,1
	INCF	FSR,1	; FSR = FSR+2
	INCF	EEADR,1	; Increase EEPROM address
	MOVLW	0x06
	XORWF	EEADR,0
	BNZ	WRITE_LOOP
	MOVLW	0x08	; Point to EEPROM address 0x08
	MOVWF	EEADR	
	MOVLW	0x01	; EEPROM data = 0x01
	MOVWF	EEDATA
	CALL	WRITE_EE	; Write indicating (0x01)
	BANKSEL	GPIO	; Return to bank 0
	CALL	SET_LINE2	; LCD line2/1
	MOVLW	.0
	MOVWF	TBL_POINTER	; Table address = 0
	MOVLW	.16
	MOVWF	NUM_OF_CHAR	; Show "Accepted ID     " on SLCD
MESSAGE6	MOVFW	TBL_POINTER
	CALL	TABLE6
	CALL	TX
	INCF	TBL_POINTER,1
	DECFSZ	NUM_OF_CHAR,1
	GOTO	MESSAGE6
	MOVLW	.10
	CALL	DELAY	; Delay 2 seconds
	GOTO	MAIN
	
MAIN	CALL	CLR_DISPLAY	; Clear LCD
	MOVLW	.0
	MOVWF	TBL_POINTER	; Table address = 0
	MOVLW	.15
	MOVWF	NUM_OF_CHAR	; Show "ELECTRONICS KEY" on SLCD
MESSAGE7	MOVFW	TBL_POINTER
	CALL	TABLE7
	CALL	TX
	INCF	TBL_POINTER,1
	DECFSZ	NUM_OF_CHAR,1
	GOTO	MESSAGE7
	CALL	SET_LINE2	; LCD LINE2/1
	MOVLW	.15
	MOVWF	TBL_POINTER	; Table address = .15
	MOVLW	.15
	MOVWF	NUM_OF_CHAR	; Show "What's your ID?" on SLCD
MESSAGE8	MOVFW	TBL_POINTER
	CALL	TABLE7
	CALL	TX
	INCF	TBL_POINTER,1
	DECFSZ	NUM_OF_CHAR,1
	GOTO	MESSAGE8
	
MAIN_LOOP	CALL	READ_ID	; Read SLAVE ID
	CALL	CRC_CAL	; CRC calculation
 	BANKSEL	EEADR	; Bank 1
	CLRF	EEADR	; Point to address 0x00
	MOVLW	0x22	; Point to ID'S TEMP
	MOVWF	FSR
COMPARE_LOOP	MOVFW	INDF
	MOVWF	ID_TEMP1
	CALL	READ_EE
	SUBWF	ID_TEMP1,0
	BNZ	ID_FAILED
	INCF	FSR,1
	INCF	FSR,1	; FSR = FSR+2
	BANKSEL	EEADR	; Bank 1
	INCF	EEADR,1
	MOVLW	0x06
	XORWF	EEADR,0	; Check EEPROM address = 6 ?
	BANKSEL	GPIO	; Bank 0
	BNZ	COMPARE_LOOP	; If EEPROM address <> 6 then compare

ID_PASSED	CALL	CLR_DISPLAY	; Clear LCD
	MOVLW	.30
	MOVWF	TBL_POINTER	; Table address = .15
	MOVLW	.9
	MOVWF	NUM_OF_CHAR	; Show "ID PASSED" on SLCD
MESSAGE10	MOVFW	TBL_POINTER
	CALL	TABLE7
	CALL	TX
	INCF	TBL_POINTER,1
	DECFSZ	NUM_OF_CHAR,1
	GOTO	MESSAGE10
	CALL	SOUND	; Send beep
	MOVLW	.10
	CALL	DELAY	; Delay 2 seconds
	GOTO	MAIN

ID_FAILED	CALL	CLR_DISPLAY	; Clear SLCD
	MOVLW	.39
	MOVWF	TBL_POINTER	; Table address = .15
	MOVLW	.10
	MOVWF	NUM_OF_CHAR	; Show "ID FAILED!" on SLCD
MESSAGE9	MOVFW	TBL_POINTER
	CALL	TABLE7
	CALL	TX
	INCF	TBL_POINTER,1
	DECFSZ	NUM_OF_CHAR,1
	GOTO	MESSAGE9
	MOVLW	.10
	CALL	DELAY	; Delay 2 seconds
	GOTO	MAIN
	
READ_EE	NOP		; Read EEPROM routine
	BANKSEL	EEADR	; Bank 1 
	BSF	EECON1,RD	; Start EEPROM read
	MOVFW	EEDATA	; Load EEPROM data to W
	BANKSEL	GPIO	; Return to bank 0
	RETURN	
WRITE_EE	BSF	EECON1,WREN	; Enable write EEPROM
	MOVLW	0x55	; EEPROM write sequence
	MOVWF	EECON2	;
	MOVLW	0xAA	;
	MOVWF	EECON2	;
	BSF	EECON1,WR	; Start write
	BTFSC	EECON1,WR	; Skip if write complete
	GOTO	$-1	; Write wait	
	BCF	EECON1,WREN	; Disable twice
	RETURN	
CONFIRM_ID1	MOVLW	.0	
	MOVWF	TBL_POINTER	; Table address = 0
	CALL	L2
	MOVLW	.15
	CALL	DELAY	; Delay 3 seconds
	CLRF	COUNT3	; COUNT3 = number of scrolling
L1	INCF	COUNT3,1
	MOVFW	COUNT3
	MOVWF	TBL_POINTER
	CALL	L2
	MOVLW	.58 
	XORWF	COUNT3,0	; COUNT3 = .58 ?
	BNZ	L1	; If no then scroll
	CALL	SET_LINE2	; LCD line 2/1
	MOVLW	.0
	MOVWF	TBL_POINTER	; Table address = 0
	MOVLW	.11
	MOVWF	NUM_OF_CHAR	; Show "Yes or No ?" on SLCD
MESSAGE4	MOVFW	TBL_POINTER
	CALL	TABLE4
	CALL	TX
	INCF	TBL_POINTER,1
	DECFSZ	NUM_OF_CHAR,1
	GOTO	MESSAGE4
	BTFSS	SW3
	GOTO	$-1	; If pressed SW3 then wait for release
	BTFSS	SW2
	GOTO	$-1	; if pressed SW2 then wait for release
	RETURN	
L2	CALL	SET_LINE2	; LCD line2/1
	MOVLW	.16
	MOVWF	NUM_OF_CHAR	; 14 characters
L3	MOVFW	TBL_POINTER
	CALL	TABLE3
	CALL	TX
	INCF	TBL_POINTER,1
	DECFSZ	NUM_OF_CHAR,1
	GOTO	L3
	CALL	DELAY200mS
	RETURN	
NO_PASSWORD	CALL	CLR_DISPLAY	; Clear SLCD
	MOVLW	.0
	MOVWF	TBL_POINTER	; Table address = 0
	MOVLW	.14
	MOVWF	NUM_OF_CHAR	; Show "No password" on SLCD
MESSAGE1	MOVFW	TBL_POINTER
	CALL	TABLE2
	CALL	TX
	INCF	TBL_POINTER,1
	DECFSZ	NUM_OF_CHAR,1
	GOTO	MESSAGE1
	MOVLW	.10
	CALL	DELAY	; Delay 2 seconds
	CALL	CLR_DISPLAY	; Clear SLCD
	MOVLW	.14
	MOVWF	TBL_POINTER	; Table address = .14
	MOVLW	.10
	MOVWF	NUM_OF_CHAR	; Show "Please set" on SLCD
MESSAGE2	MOVFW	TBL_POINTER
	CALL	TABLE2
	CALL	TX
	INCF	TBL_POINTER,1
	DECFSZ	NUM_OF_CHAR,1
	GOTO	MESSAGE2
	CALL	SET_LINE2	; Set LCD LINE2/1
	MOVLW	.25
	MOVWF	TBL_POINTER	; Table address = .25
	MOVLW	.13
	MOVWF	NUM_OF_CHAR	; Show "your password" on SLCD
MESSAGE3	MOVFW	TBL_POINTER
	CALL	TABLE2
	CALL	TX
	INCF	TBL_POINTER,1
	DECFSZ	NUM_OF_CHAR,1
	GOTO	MESSAGE3
	RETURN		
	
CRC_CAL	CLRF	CRC	; Start calculate with CRC = 00
	MOVLW	0x21 	; Point to family code <COPY>
	MOVWF	FSR
CRC_LOOP1	MOVLW	.8	; Number of rotate loop
	MOVWF	COUNT1		
CRC_LOOP2	MOVFW	INDF	; Load data to W
	XORWF	CRC,0	
	MOVWF	COUNT2
	RRF	COUNT2,1
	BNC	NO_ADJ	; If C=0 then do not adjust
	MOVLW	B'00011000'	
	XORWF	CRC,1	; Toggle bit 4 and bit 3
NO_ADJ	RRF	CRC,1	
	RRF	INDF,1
	DECFSZ	COUNT1,1
	GOTO	CRC_LOOP2
	INCF	FSR,1	; Increase indirect address
	INCF	FSR,1	; FSR = FSR+2
	MOVLW	0x2F
	SUBWF	FSR,0	; Pointer = 0x2F ?
	BNZ	CRC_LOOP1	; No, calculation again
	RETURN		; Yes, end of job

READ_ID	CALL	RESET_PULSE	; TX reset pulse
	CALL	PRES_PULSE	; RX presence pulse
	MOVLW	0x33	; Send read ROM command for DS1990A 
	CALL	OW_WRITE
	CALL	DELAY15U	; delay 15 microsecond
	MOVLW	.8	; For read 8 times (read ID)
	MOVWF	TEMP
	MOVLW	0x20	; Set FSR address
	MOVWF	FSR	; FSR = address pointer
READ_ID_LOOP	CALL	OW_READ
	MOVWF	INDF
	INCF	FSR,1	; Increase pointer
	MOVWF	INDF	; Copy ID
	INCF	FSR,1	; Increment pointer 
	DECFSZ	TEMP,1	; Skip if completed <8 bytes>
	GOTO	READ_ID_LOOP	; Read again...
CHECK_00_?	MOVLW	.7	; Check data = 00 ?
	MOVWF	TEMP	
	MOVLW	0x21
	MOVWF	FSR	; Point to ID
	MOVFW	INDF	; W = ID
	MOVWF	COUNT1
LOOP_00	INCF	FSR,1
	INCF	FSR,1	; Point to next ID
	MOVFW	INDF
	IORWF	COUNT1,1	; COUNT1 = COUNT1 OR W
	DECFSZ	TEMP,1
	GOTO	LOOP_00
	TSTF	COUNT1	; COUNT1 = 00 ?
	BZ	READ_ID	; Yes, read again
CHECK_FF_?	MOVLW	.7
	MOVWF	TEMP	; Check data = FF ?
	MOVLW	0x21
	MOVWF	FSR	; Point to ID
	MOVFW	INDF	; W = ID
	MOVWF	COUNT1
LOOP_FF	INCF	FSR,1
	INCF	FSR,1	; Point to next ID
	MOVFW	INDF
	ANDWF	COUNT1,1	; COUNT1 = COUNT1 AND W
	DECFSZ	TEMP,1
	GOTO	LOOP_FF
	MOVLW	0xFF
	XORWF	COUNT1,0	; COUNT1 = FF ?
	BZ	READ_ID	; Yes, read again
 	RETURN

SHOW	CALL	SET_LINE1	; LCD line1/1
	MOVLW	"I"	; Show "ID= "
	CALL	TX	;
	MOVLW	"D"	;
	CALL	TX	;
	MOVLW	"="	;
	CALL	TX	;
	MOVLW	" "	;
	CALL	TX	;
	MOVLW	.6	; For showing ID 6 bytes
	MOVWF	TEMP	
	MOVLW	0x2C 	; Set FSR address that stored already
	MOVWF	FSR	; FSR = pointer 
SHOW_LOOP	MOVFW	INDF
	CALL	SHOW_ID
	DECF	FSR,1	; Decrease indirect address
	DECF	FSR,1	; FSR = FSR-2
	DECFSZ	TEMP,1	; Skip if completed
	GOTO	SHOW_LOOP
	RETURN
SHOW_ID	MOVWF	D_TEMP	; Find & show 0-F 
	SWAPF	D_TEMP,0	; Exchanged data 
	ANDLW	B'00001111'	; Mask bit
	CALL	TABLE1	; Find number of digit1
	CALL	TX	; Send digit1 to SLCD
	MOVFW	D_TEMP
	ANDLW	B'00001111'	; Mask bit
	CALL	TABLE1	; Find number of digit2
	CALL	TX	; Send digit2 to SLCD
	RETURN	
RESET_PULSE	BCF	DQ_PIN  	; Logic 0, TX reset pulse
	BSF	STATUS,RP0	; Bank 1
	BCF	DQ_PIN 	; Port = output
	BCF	STATUS,RP0	; Bank 0
	CALL	DELAY617U	; For reset pulse
	BSF	STATUS,RP0	; Bank 1
	BSF	DQ_PIN 	; Port = high impedance
	BCF	STATUS,RP0 	; Bank 0 
	RETURN

PRES_PULSE    BSF	STATUS,RP0	; Bank 1,RX presence pulse
	BSF	DQ_PIN 	; Port = high impedance
	BCF	STATUS,RP0	; Bank 0 
	CALL	DELAY15U	; delay 15 microsecond
	BTFSC	DQ_PIN 	; Skip if clear
	GOTO	$-1	; wait for presence pulse
	BTFSS	DQ_PIN 	; Skip if DQ = idle
	GOTO	$-1	; wait for 1-wire idel state
	RETURN
	
OW_WRITE	MOVWF	TEMP_WR	; 1-wire write data 
	MOVLW	.8	; Write loop 8 times
	MOVWF	COUNT8
WR_LOOP	RRF	TEMP_WR,1
	BC	WR_1	; Branch if C = 1
WR_0	BCF	DQ_PIN 	; DQ = 0
	BSF	STATUS,RP0	; Bank 1
	BCF	DQ_PIN 	; Port = output
	BCF	STATUS,RP0	; Bank 0
	CALL	DELAY15U	; Delay 15 microsecond
	CALL	DELAY45U	; Delay 45 microsecond
	BSF	STATUS,RP0	; Bank 1
	BSF	DQ_PIN 	; Port = float
	BCF	STATUS,RP0	; Bank 0
	NOP		; Delay 1 microsecond
	GOTO	$+1	; Delay 2 microsecond
	DECFSZ	COUNT8,1	; Skip if completed
	GOTO	WR_LOOP
	RETURN	
WR_1	BCF	DQ_PIN 	; DQ = 0
	BSF	STATUS,RP0	; Bank 1
	BCF	DQ_PIN 	; Port = output
	BCF	STATUS,RP0	; Bank 0
	CALL	DELAY15U	; Delay 15 microsecond
	BSF	STATUS,RP0	; Bank 1
	BSF	DQ_PIN 	; Port = float
	BCF	STATUS,RP0	; Bank 0
	CALL	DELAY45U	; Delay 45 microsecond
	NOP		; Delay 1 microsecond
	GOTO	$+1	; Delay 2 microsecond
	DECFSZ	COUNT8,1	; Skip if completed
	GOTO	WR_LOOP
	RETURN		

OW_READ	MOVLW	.8
	MOVWF	COUNT8	
READ_LOOP	BCF	DQ_PIN 
	BSF	STATUS,RP0	; Bank 1
	BCF	DQ_PIN 	; Port = output
	BCF	STATUS,RP0	; Bank 0
	GOTO	$+1	; delay 2 microsecond
	GOTO	$+1	; delay 2 microsecond
	BSF	STATUS,RP0	; Bank 1
	BSF	DQ_PIN 	; Port = inPUT
	BCF	STATUS,RP0	; Bank 0
READ_LOOP2	BTFSC	DQ_PIN 	; Read data from DQ pin
	GOTO	READ_1
READ_0	CLRC		; Carry = 0
READ_1_2	RRF	TEMP_RD,1	
	CALL	DELAY45U
	DECFSZ	COUNT8,1
	GOTO	READ_LOOP 
	MOVFW	TEMP_RD	; W = data
	RETURN	
READ_1	SETC		; Carry = 1
	GOTO	READ_1_2

TX	MOVWF	TX_BUFFER	; 8N1,9600 bps,not invert
	MOVLW	.8
	MOVWF	COUNT1
	BCF	TX_PIN	; Start bit
	NOP		; 1 microsecond
	GOTO	$+1	; 2 microsecond
	CALL	DELAY_TX	; 2 microsecond
LOOP_SEND	RRF	TX_BUFFER,1	; Send LSB first
	SKPNC
	GOTO	SEND_1
SEND_0	NOP
	BCF	TX_PIN	; Logic 0
SEND_1_2	CALL	DELAY_TX	; 2 microsecond
	DECFSZ	COUNT1,1
	GOTO	LOOP_SEND		
	BSF	TX_PIN	; Stop bit <HIGH>
	NOP
	GOTO	$+1
	CALL	DELAY_TX
	RETURN	
SEND_1	BSF	TX_PIN	; Logic 1
	GOTO	SEND_1_2

CLR_DISPLAY	MOVLW	0xFE	; SLCD command prefix
	CALL	TX
	MOVLW	0x01	; Clear display command
	CALL	TX
	RETURN
SET_LINE1	MOVLW	0xFE	; Set SLCD line 1 column 1
	CALL	TX
	MOVLW	0x80
	CALL	TX
	RETURN
SET_LINE2	MOVLW	0xFE	; Set SLCD line 2 column 1
	CALL	TX
	MOVLW	0xC0
	CALL	TX
	RETURN
SOUND	MOVLW	.125	; Sound loop
	MOVWF	SOUND_TEMP
SOUND_LOOP	BSF	GPIO,0
	MOVLW	.25	; Sound delay for logic "1"
	CALL	DELAY_SOUND
	BCF	GPIO,0
	MOVLW	.100 	; Sound delay for logic "0"
	CALL	DELAY_SOUND
	DECFSZ	SOUND_TEMP,1	; Skip if end of sound loop
	GOTO	SOUND_LOOP
	RETURN
DELAY_SOUND	MOVWF	D_VAR1	
	DECFSZ	D_VAR1,1
	GOTO	$-1
	RETURN	
;***************************
; DELAY SUBROUTINES 
;***************************
DELAY_TX	MOVLW	.29	; Delay for RS-232
	MOVWF	D_VAR1
	DECFSZ	D_VAR1,1
	GOTO	$-1
	RETURN
DELAY15U	MOVLW	.5	; Delay 15 microsecond
	MOVWF	D_VAR1
	DECFSZ	D_VAR1,1
	GOTO	$-1
	RETURN
DELAY45U	MOVLW	.15           ; Delay 45 microsecond
	MOVWF	D_VAR1
	DECFSZ	D_VAR1,1
	GOTO	$-1
	NOP
	RETURN	
DELAY51	MOVLW	.16	; Delay 51 microsecond for half bit
	MOVWF	D_VAR1	
	DECFSZ	D_VAR1,1
	GOTO	$-1	
	RETURN			
DELAY617U	MOVLW	.206	; Delay 617 microsecond
	MOVWF	D_VAR1
	DECFSZ	D_VAR1,1
	GOTO	$-1
	RETURN

DELAY	MOVWF	D_VAR3
	CLRF	D_VAR2
	CLRF	D_VAR1
	DECFSZ	D_VAR1,1
	GOTO	$-1
	DECFSZ	D_VAR2,1
	GOTO	$-3
	DECFSZ	D_VAR3,1
	GOTO	$-5
	RETURN
DELAY200mS	MOVLW	.1	; Delay 200 millisecond for scrolling
	CALL	DELAY
	RETURN
	
	END

